# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.shortcuts import get_object_or_404
from django.http import HttpResponseRedirect,HttpResponseForbidden,HttpResponse
from django import forms

from noc.peer.models import Person,Maintainer,AS,ASSet,PeeringPoint
from noc.lib.render import render,render_plain_text,render_json
from noc.lib.validators import is_asn,is_as_set,is_ipv4,is_cidr,is_fqdn
import re

def as_rpsl(request,asn):
    assert is_asn(asn)
    asn=get_object_or_404(AS,asn=int(asn))
    return render_plain_text(asn.rpsl)
    
def as_dot(request,asn):
    assert is_asn(asn)
    asn=get_object_or_404(AS,asn=int(asn))
    return render(request,"peer/as_dot.html",{"asn":asn})

def as_set_rpsl(request,as_set):
    assert is_as_set(as_set)
    as_set=get_object_or_404(ASSet,name=as_set)
    return render_plain_text(as_set.rpsl)
    
def inet_rtr_rpsl(request,router):
    assert is_fqdn(router)
    peering_point=get_object_or_404(PeeringPoint,hostname=router)
    return render_plain_text(peering_point.rpsl)

def person_rpsl(request,person_id):
    person=get_object_or_404(Person,id=int(person_id))
    return render_plain_text(person.rpsl)

def maintainer_rpsl(request,mnt_id):
    maintainer=get_object_or_404(Maintainer,id=int(mnt_id))
    return render_plain_text(maintainer.rpsl)
##
## Looking glass
##
#class LGForm(forms.Form):
#    peering_point= forms.ModelChoiceField(queryset=PeeringPoint.objects.filter(lg_rcmd__isnull=False).exclude(lg_rcmd__exact=""))
#    query_type   = forms.ModelChoiceField(queryset=LGQueryType.objects.all())
#    query        = forms.CharField(required=False)
#    def clean_query(self):
#        peering_point = self.cleaned_data.get("peering_point", None)
#        query_type = self.cleaned_data.get("query_type", None)
#        query = self.cleaned_data.get("query", "").strip()
#        if peering_point and query_type:
#            try:
#                qc=LGQueryCommand.objects.get(profile_name=peering_point.profile_name,query_type=query_type)
#            except LGQueryCommand.DoesNotExist:
#                raise forms.ValidationError("Query type is not supported for this router")
#            if query=="" and qc.is_argument_required:
#                raise forms.ValidationError("Missed query argument")
#            if query!="" and (not is_ipv4(query) and not is_cidr(query)):
#                raise forms.ValidationError("Invalid query")
#        return query
    
def lg(request):
    task_id=None
    if request.POST:
        form=LGForm(request.POST)
        if form.is_valid():
            pp=form.cleaned_data["peering_point"]
            cmd=pp.lg_command(form.cleaned_data["query_type"],form.cleaned_data["query"])
            task_id=Task.create_task(
                pp.profile_name,
                pp.lg_rcmd,
                "sa.actions.cli",
                args={"commands":[cmd]},
                timeout=60
            )
    else:
        form=LGForm()
    return render(request,"peer/lg.html",{"form":form,"task_id":task_id})
    
def lg_json(request,task_id):
    t=get_object_or_404(Task,task_id=int(task_id))
    # Format output
    out=t.out
    profile=t.profile
    if profile.pattern_lg_as_path_list:
        rx=re.compile(profile.pattern_lg_as_path_list,re.DOTALL|re.MULTILINE)
        out=rx.sub(profile.lg_as_path,out)
    if profile.pattern_lg_best_path:
        rx=re.compile(profile.pattern_lg_best_path,re.DOTALL|re.MULTILINE)
        out=rx.sub(r"<span style='color: red'>\1</span>",out)
    r={
        "status" : t.status,
        "out"    : out,
    }
    # Remove complete and failed queries
    if t.status in ["c","f"]:
        t.delete()
    return render_json(r)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *

class Migration:
    
    def forwards(self):
        db.add_column("cm_objectcategory","notify_immediately",models.TextField("Notify Immediately",blank=True,null=True))
        db.add_column("cm_objectcategory","notify_delayed",models.TextField("Notify Delayed",blank=True,null=True))
            
    def backwards(self):
        db.delete_column("cm_objectcategory","notify_immediately")
        db.delete_column("cm_objectcategory","notify_delayed")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## 802.11 WI-FI events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var

##
## A station associated to an access point.
##
class Dot11Associated(EventClass):
    name="dot11 Associated"
    category="NETWORK"
    priority="INFO"
    subject_template="dot11 station associated: {{mac}}"
    body_template="dot11 station associated: {{mac}}"
    class Vars:
        mac=Var(required=True)

##
## A station disassociated from an access point.
##
class Dot11Disassociated(EventClass):
    name="dot11 Disassociated"
    category="NETWORK"
    priority="INFO"
    subject_template="dot11 station disassociated: {{mac}}"
    body_template="dot11 station disassociated: {{mac}}"
    class Vars:
        mac=Var(required=True)
##
## A station has roamed to a new access point.
##
class Dot11Roamed(EventClass):
    name="dot11 Roamed"
    category="NETWORK"
    priority="INFO"
    subject_template="dot11 station roamed: {{mac}}"
    body_template="dot11 station roamed: {{mac}}"
    class Vars:
        mac=Var(required=True)
##
##
##
class Dot11Deauthenticate(EventClass):
    name="dot11 Deauthenticate"
    category="NETWORK"
    priority="INFO"
    subject_template="dot11 deauthenticate: {{mac}}"
    body_template="dot11 deauthenticate: {{mac}}"
    class Vars:
        mac=Var(required=True)
##
## Too many retries to client
##
class Dot11MaxRetries(EventClass):
    name="dot11 Max Retries"
    category="NETWORK"
    priority="INFO"
    subject_template="dot11 Too Many Retries: {{mac}}"
    body_template="dot11 Too many retries: {{mac}}. Removing the client"
    class Vars:
        mac=Var(required=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django URL dispatcher.
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.conf.urls.defaults import *
from django.contrib import admin
admin.autodiscover()

handler404="noc.main.views.handler404"

urlpatterns = patterns('',
    # For debugging purposes only. Overriden by lighttpd/apache
     (r'^static/(?P<path>.*)$', 'django.views.static.serve', {'document_root': 'static'}),
     (r'^doc/(?P<path>.*)$',    'django.views.static.serve', {'document_root': 'share/doc/users_guide/html/'}),
    # 
     (r'^$',      include('noc.main.urls')),
     (r'^admin/(.*)', admin.site.root),
     (r'^accounts/login/$', 'django.contrib.auth.views.login', {'template_name':'template/login.html'}),
     (r'^main/',  include('noc.main.urls')),
     (r'^sa/',  include('noc.sa.urls')),
     (r'^fm/', include('noc.fm.urls')),
     (r'^cm/',  include('noc.cm.urls')),
     (r"^ip/", include("noc.ip.urls")),
     (r"^vc/", include("noc.vc.urls")),
     (r"^peer/", include("noc.peer.urls")),
     (r"^dns/", include("noc.dns.urls")),
)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.MatrixReport):
    name="sa.objects_by_profiles_and_domains"
    title="Objects by Profile and Domains"
    requires_cursor=True
    
    def get_queryset(self):
        return self.execute("SELECT d.name,profile_name,COUNT(*) FROM sa_managedobject o JOIN sa_administrativedomain d ON (o.administrative_domain_id=d.id) GROUP BY 1,2")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'EventRepeat'
        db.create_table('fm_eventrepeat', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('event', models.ForeignKey(Event,verbose_name="Event")),
            ('timestamp', models.DateTimeField("Timestamp"))
        ))
        # Mock Models
        EventClass = db.mock_model(model_name='EventClass', db_table='fm_eventclass', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventClassVar'
        db.create_table('fm_eventclassvar', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('event_class', models.ForeignKey(EventClass,verbose_name="Event Class")),
            ('name', models.CharField("Name",max_length=64)),
            ('required', models.BooleanField("Required",default=True)),
            ('repeat_suppression', models.BooleanField("Repeat Suppression",default=False))
        ))
        db.create_index('fm_eventclassvar', ['event_class_id','name'], unique=True, db_tablespace='')
        db.send_create_signal('fm', ['EventRepeat','EventClassVar'])
        
        db.add_column('fm_eventclass','repeat_suppression',models.BooleanField("Repeat Suppression",default=False))
        db.add_column('fm_eventclass','repeat_suppression_interval',models.IntegerField("Repeat Suppression interval (secs)",default=3600))
        # Migrate variables
        for id,vars in db.execute("SELECT id,variables FROM fm_eventclass"):
            if vars:
                for v in [v.strip() for v in vars.split(",")]:
                    db.execute("INSERT INTO fm_eventclassvar(event_class_id,name,required,repeat_suppression) VALUES(%s,%s,true,false)",[id,v])
        db.delete_column('fm_eventclass','variables')        
    
    def backwards(self):
        db.delete_table('fm_eventrepeat')
        db.delete_column('fm_eventclass','repeat_suppression')
        db.delete_column('fm_eventclass','repeat_suppression_interval')
        db.delete_table('fm_eventclassvar')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.db import models
from noc.main.report import report_registry
from noc.main.menu import populate_reports
from noc.main.menu import Menu

##
## Application Menu
##
class AppMenu(Menu):
    app="main"
    title="Main"
    items=[
        ("Setup", [
            ("Users",  "/admin/auth/user/",  "auth.change_user"),
            ("Groups", "/admin/auth/group/", "auth.change_group")
        ])
    ]
##
## Load and register reports
##
report_registry.register_all()
populate_reports([(n,r.title) for n,r in report_registry.classes.items()])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report

class Report(noc.main.report.Report):
    name="dns.ns_zones"
    title="Zones at nameservers"
    requires_cursor=True
    columns=[Column("Nameserver"),Column("Master Zones",align="RIGHT"),Column("Slave Zones",align="RIGHT")]
    
    def get_queryset(self):
        return self.execute("""SELECT ns.name,
                                (
                                    SELECT COUNT(*)
                                    FROM dns_dnszoneprofile_masters pm JOIN dns_dnszone z ON (z.profile_id=pm.dnszoneprofile_id)
                                    WHERE pm.dnsserver_id=ns.id AND z.is_auto_generated
                                ) AS masters,
                                (
                                    SELECT COUNT(*)
                                    FROM dns_dnszoneprofile_slaves ps JOIN dns_dnszone z ON (z.profile_id=ps.dnszoneprofile_id)
                                    WHERE ps.dnsserver_id=ns.id AND z.is_auto_generated
                                ) AS slaves
                            FROM dns_dnsserver ns
                            ORDER BY 2 DESC, 1 ASC""")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetConfig

class Script(noc.sa.script.Script):
    name="Protei.MediaGateway.get_config"
    implements=[IGetConfig]
    def execute(self):
        self.cli("cd /usr/protei/CLI/Client")
        self.cli("./clip")
        for i in range(5):
            r=self.cli("end")
            if "parent." in r:
                break
        self.cli("show-recursive")
        config=self.cli("")
        return self.cleaned_config(config)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
from noc.lib.validators import is_ipv4
from noc.cm.models import Config

class Report(noc.main.report.Report):
    name="cm.ip_addresses_in_config"
    title="IP Addresses in Config"
    columns=[
            Column("Repo Path"),
            Column("IP Address")
            ]
    
    def get_queryset(self):
        return [(c.repo_path,c.address) for c in Config.objects.all() if is_ipv4(c.address)]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
import os

class Report(noc.main.report.Report):
    name="main.apps_loc"
    title="Lines of code"
    requires_cursor=False
    columns=[Column("App"),
            Column(".py lines",align="RIGHT"),
            Column(".html lines",align="RIGHT"),
            Column(".mib lines",align="RIGHT"),
            Column("Other lines",align="RIGHT")]
    
    def get_queryset(self):
        def loc(f):
            f=open(f)
            d=f.read()
            f.close()
            return len(d.split("\n"))
        r=[]
        for dirname in [d for d in os.listdir(".") if os.path.isdir(d) and not d.startswith(".")]:
            py_loc=0
            html_loc=0
            mib_loc=0
            other_loc=0
            for dirpath,dirnames,filenames in os.walk(dirname):
                for f in [f for f in filenames if not (f.endswith(".pyc") or f.endswith(".pyo"))]:
                    lines=loc(os.path.join(dirpath,f))
                    if f.endswith(".py"):
                        py_loc+=lines
                    elif f.endswith(".html"):
                        html_loc+=lines
                    elif f.endswith(".mib"):
                        mib_loc+=lines
                    else:
                        other_loc+=lines
            r.append((dirname,py_loc,html_loc,mib_loc,other_loc))
        return r
            



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Juniper.JUNOS Configuraion-related rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.config import *
##
## Juniper.JUNOS Config Changed SNMP
##
class Juniper_JUNOS_Config_Changed_SNMP_Rule(ClassificationRule):
    name="Juniper.JUNOS Config Changed SNMP"
    event_class=ConfigChanged
    preference=1000
    required_mibs=["JUNIPER-CFGMGMT-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^profile$",r"^Juniper\.JUNOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.2636\.4\.5\.0\.1$"), # jnxCmCfgChange
    ]
##
## Juniper.JUNOS Config Changed CM SNMP
##
class Juniper_JUNOS_Config_Changed_CM_SNMP_Rule(ClassificationRule):
    name="Juniper.JUNOS Config Changed CM SNMP"
    event_class=ConfigChanged
    preference=1000
    required_mibs=["JUNIPER-CFGMGMT-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^profile$",r"^Juniper\.JUNOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.2636\.4\.5$"),
        (r"^1\.3\.6\.1\.4\.1\.2636\.3\.18\.1\.7\.1\.5\.\d+$",r"^(?P<user>.*)$"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco IOS DHCP rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.dhcp import *
##
## Cisco.IOS DHCPD Address Conflict SYSLOG
##
class Cisco_IOS_DHCPD_Address_Conflict_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS DHCPD Address Conflict SYSLOG"
    event_class=DHCPDAddressConflict
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^severity$",r"^4$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%DHCPD-4-PING_CONFLICT: DHCP address conflict:  server pinged (?P<ip>\S+)\.$"),
    ]
##
## Cisco.IOS DHCPD Address Conflict SYSLOG SNMP
##
class Cisco_IOS_DHCPD_Address_Conflict_SYSLOG_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS DHCPD Address Conflict SYSLOG SNMP"
    event_class=DHCPDAddressConflict
    preference=1000
    require_mibs=["CISCO-SYSLOG-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.2\.0\.1$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.5\.\d+$",r"(?P<ip>\S+)\.$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.4\.\d+$",r"^PING_CONFLICT$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.2\.\d+$",r"^DHCPD$"),
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.shortcuts import get_object_or_404
from noc.lib.render import render
from noc.sa.models import ManagedObject,script_registry,profile_registry
from django.http import HttpResponseForbidden,HttpResponseNotFound
from xmlrpclib import ServerProxy, Error
from noc.settings import config
import pprint,types

def object_scripts(request,object_id):
    o=get_object_or_404(ManagedObject,id=int(object_id))
    if not o.has_access(request.user):
        return HttpResponseForbidden("Access denied")
    p=o.profile_name
    scripts=sorted([(p+"."+x,x) for x in profile_registry[p].scripts.keys()])
    return render(request,"sa/scripts.html",{"object":o,"scripts":scripts})

def object_script(request,object_id,script):
    def get_result(script,object_id,**kwargs):
        server=ServerProxy("http://%s:%d"%(config.get("xmlrpc","server"),config.getint("xmlrpc","port")))
        result=server.script(script,object_id,kwargs)
        if type(result) not in [types.StringType,types.UnicodeType]:
            result=pprint.pformat(result)
        return result
        
    o=get_object_or_404(ManagedObject,id=int(object_id))
    if not o.has_access(request.user):
        return HttpResponseForbidden("Access denied")
    try:
        scr=script_registry[script]
    except:
        return HttpResponseNotFound("No script found")
    form=None
    result=None
    if scr.implements and scr.implements[0].requires_input():
        if request.POST:
            form=scr.implements[0].get_form(request.POST)
            if form.is_valid():
                data={}
                for k,v in form.cleaned_data.items():
                    if v:
                        data[k]=v
                result=get_result(script,object_id,**data)
        else:
            form=scr.implements[0].get_form()
    else:
        result=get_result(script,object_id)
    return render(request,"sa/script.html",{"object":o,"result":result,"script":script,"form":form})



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.registry import Registry
##
##
##
class GeneratorRegistry(Registry):
    name="GeneratorRegistry"
    subdir="generators"
    classname="Generator"
    apps=["noc.dns"]
generator_registry=GeneratorRegistry()
##
## Metaclass for Generator
##
class GeneratorBase(type):
    def __new__(cls,name,bases,attrs):
        g=type.__new__(cls,name,bases,attrs)
        generator_registry.register(g.name,g)
        return g

##
## Abstract DNS zone generator
##
class Generator(object):
    __metaclass__=GeneratorBase
    name=None
    def get_header(self):
        return """;;
;; WARNING: Auto-generated zone file
;; Do not edit manually
;; Use python manage.py sync-zones to rebuild
;;
"""
    def get_footer(self):
        return """
;;
;; End of auto-generated zone
;;
"""
    def get_soa(self):
        raise Exception,"SOA Not implemented"
    
    def get_records(self):
        raise Exception,"Records Not implemented"
        
    def get_zone(self,zone):
        self.zone=zone
        s=""
        s+=self.get_header()
        s+=self.get_soa()
        s+=self.get_records()
        s+=self.get_footer()
        return s
        
    def get_include(self,ns):
        raise Exception,"Include file is not implemented"
        
    def format_3_columns(self,records):
        maxlen_1=10
        maxlen_2=3
        records=[r for r in records if len(r)==3]
        for a,b,c in records:
            l=len(a)
            if l>maxlen_1:
                maxlen_1=l
            l=len(b)
            if l>maxlen_2:
                maxlen_2=l
        mask="%%-%ds   %%-%ds   %%s"%(maxlen_1,maxlen_2)
        return "\n".join([mask%tuple(r) for r in records if len(r)==3])
    
    def pretty_time(self,t):
        if t==0:
            return "zero"
        T=["week","day","hour","min","sec"]
        W=[345600,86400,3600,60,1]
        r=[]
        for w in W:
            rr=int(t/w)
            t-=rr*w
            r.append(rr)
        z=[]
        for rr,t in zip(r,T):
            if rr>1:
                z.append("%d %ss"%(rr,t))
            elif rr>0:
                z.append("%d %s"%(rr,t))
        return " ".join(z)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## on_config_changed
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.fm.triggers
import datetime
##
## on_config_changed trigger sets next_pull of next config check to
## event time +10 minutes. If resulting next_pull points to past
## next pull will left unaltered
##
class EventTrigger(noc.fm.triggers.EventTrigger):
    name="on_config_changed"
    def handle(self,event):
        next_pull=event.timestamp+datetime.timedelta(minutes=10)
        if next_pull<datetime.datetime.now(): # next_pull points to the past
            return
        try:
            config=event.managed_object.config
        except:
            return # No config found
        config.next_pull=next_pull
        config.save()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.dns.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'DNSZoneProfile'
        db.create_table('dns_dnszoneprofile', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('zone_transfer_acl', models.CharField("named zone transfer ACL",max_length=64)),
            ('zone_ns_list', models.CharField("NS list",max_length=64)),
            ('zone_soa', models.CharField("SOA",max_length=64)),
            ('zone_contact', models.CharField("Contact",max_length=64)),
            ('zone_refresh', models.IntegerField("Refresh",default=3600)),
            ('zone_retry', models.IntegerField("Retry",default=900)),
            ('zone_expire', models.IntegerField("Expire",default=86400)),
            ('zone_ttl', models.IntegerField("TTL",default=3600))
        ))
        
        # Mock Models
        DNSZoneProfile = db.mock_model(model_name='DNSZoneProfile', db_table='dns_dnszoneprofile', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'DNSZone'
        db.create_table('dns_dnszone', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Domain",max_length=64,unique=True)),
            ('description', models.CharField("Description",null=True,blank=True,max_length=64)),
            ('is_auto_generated', models.BooleanField("Auto generated?")),
            ('serial', models.CharField("Serial",max_length=10,default="0000000000")),
            ('profile', models.ForeignKey(DNSZoneProfile,verbose_name="Profile"))
        ))
        # Model 'DNSZoneRecordType'
        db.create_table('dns_dnszonerecordtype', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('type', models.CharField("Type",max_length=16,unique=True)),
        ))
        
        # Mock Models
        DNSZone = db.mock_model(model_name='DNSZone', db_table='dns_dnszone', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        DNSZoneRecordType = db.mock_model(model_name='DNSZoneRecordType', db_table='dns_dnszonerecordtype', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'DNSZoneRecord'
        db.create_table('dns_dnszonerecord', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('zone', models.ForeignKey(DNSZone,verbose_name="Zone")),
            ('left', models.CharField("Left",max_length=32,blank=True,null=True)),
            ('type', models.ForeignKey(DNSZoneRecordType,verbose_name="Type")),
            ('right', models.CharField("Right",max_length=64))
        ))
        
        db.send_create_signal('dns', ['DNSZoneProfile','DNSZone','DNSZoneRecordType','DNSZoneRecord'])
    
    def backwards(self):
        db.delete_table('dns_dnszonerecord')
        db.delete_table('dns_dnszonerecordtype')
        db.delete_table('dns_dnszone')
        db.delete_table('dns_dnszoneprofile')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'TaskSchedule'
        db.create_table('sa_taskschedule', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('periodic_name', models.CharField("Periodic Task",max_length=64,choices=periodic_registry.choices)),
            ('is_enabled', models.BooleanField("Enabled?",default=False)),
            ('run_every', models.PositiveIntegerField("Run Every (secs)",default=86400)),
            ('retries', models.PositiveIntegerField("Retries",default=1)),
            ('retry_delay', models.PositiveIntegerField("Retry Delay (secs)",default=60)),
            ('timeout', models.PositiveIntegerField("Timeout (secs)",default=300)),
            ('next_run', models.DateTimeField("Next Run",auto_now_add=True)),
            ('retries_left', models.PositiveIntegerField("Retries Left",default=1))
        ))
        
        db.send_create_signal('sa', ['TaskSchedule'])
    
    def backwards(self):
        db.delete_table('sa_taskschedule')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *
from noc.settings import config
import os,stat,datetime

TYPES={
    "config":"config",
    "prefixlist":"prefix-list",
    "dns":"dns",
    "rpsl":"rpsl",
    }

class Migration: 
    def forwards(self):
        repo_root=config.get("cm","repo")
        for ot in TYPES:
            db.add_column("cm_%s"%ot,"last_modified",models.DateTimeField("Last Modified",blank=True,null=True))
            if repo_root:
                repo=os.path.join(repo_root,TYPES[ot])
                for id,repo_path in db.execute("SELECT id,repo_path FROM cm_%s"%ot):
                    path=os.path.join(repo,repo_path)
                    if os.path.exists(path):
                        lm=datetime.datetime.fromtimestamp(os.stat(path)[stat.ST_MTIME])
                        db.execute("UPDATE cm_%s SET last_modified=%%s WHERE id=%%s"%ot,[lm,id])
    
    def backwards(self):
        for ot in TYPES:
            db.delete_column("cm_%s"%ot,"last_modified")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVersion
import re

rx_ver=re.compile(r"^Cisco IOS Software, (?P<platform>[^ ]+) Software \((?P<image>[^)]+)\), Version (?P<version>[^,]+),",re.MULTILINE|re.DOTALL)

class Script(noc.sa.script.Script):
    name="Cisco.IOS.get_version"
    implements=[IGetVersion]
    def execute(self):
        self.cli("terminal length 0")
        v=self.cli("show version")
        match=rx_ver.search(v)
        return {
            "vendor"    : "Cisco",
            "platform"  : match.group("platform"),
            "version"   : match.group("version"),
            "image"     : match.group("image"),
        }



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Abstract script interfaces
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import re,types

try:
    from django import forms
except ImportError:
    # No django. Interface.get_form() is meaningless
    pass
##
##
##
class InterfaceTypeError(Exception): pass
##
## Abstract parameter
##
class Parameter(object):
    def __init__(self,required=True,default=None):
        self.required=required
        if default:
            default=self.clean(default)
        self.default=default
        
    def clean(self,value):
        raise InterfaceTypeError
        
    def get_form_field(self):
        return forms.CharField(required=self.required,initial=self.default)
##
##
##
class NoneParameter(Parameter):
    """
    >>> NoneParameter().clean(None)
    >>> NoneParameter().clean("None")
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    """
    def __init__(self,required=True):
        super(NoneParameter,self).__init__(required=required)
    def clean(self,value):
        if value is not None:
            raise InterfaceTypeError
        return value
##
##
##
class StringParameter(Parameter):
    """
    >>> StringParameter().clean("Test")
    'Test'
    >>> StringParameter().clean(10)
    '10'
    >>> StringParameter().clean(None)
    'None'
    """
    def clean(self,value):
        try:
            return str(value)
        except:
            raise InterfaceTypeError
##
##
##
class REStringParameter(StringParameter):
    """
    >>> REStringParameter("ex+p").clean("exp")
    'exp'
    >>> REStringParameter("ex+p").clean("exxp")
    'exxp'
    >>> REStringParameter("ex+p").clean("regexp 1")
    'regexp 1'
    >>> REStringParameter("ex+p").clean("ex")
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    """
    def __init__(self,regexp,required=True,default=None):
        super(REStringParameter,self).__init__(required=required,default=default)
        self.rx=re.compile(regexp)

    def clean(self,value):
        v=super(REStringParameter,self).clean(value)
        if not self.rx.search(v):
            raise InterfaceTypeError
        return v
##
##
##
class BooleanParameter(Parameter):
    """
    >>> BooleanParameter().clean(True)
    True
    >>> BooleanParameter().clean(False)
    False
    >>> BooleanParameter().clean("True")
    True
    >>> BooleanParameter().clean("yes")
    True
    >>> BooleanParameter().clean(1)
    True
    >>> BooleanParameter().clean(0)
    False
    >>> BooleanParameter().clean([])
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    """
    def clean(self,value):
        if type(value)==types.BooleanType:
            return value
        if type(value) in [types.IntType,types.LongType]:
            return value!=0
        if type(value) in [types.StringType,types.UnicodeType]:
            return value.lower() in ["true","t","yes","y"]
        raise InterfaceTypeError
##
##
##
class IntParameter(Parameter):
    """
    >>> IntParameter().clean(1)
    1
    >>> IntParameter().clean("1")
    1
    >>> IntParameter().clean("not a number")
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    >>> IntParameter(min_value=10).clean(5)
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    >>> IntParameter(max_value=7).clean(10)
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    """
    def __init__(self,required=True,default=None,min_value=None,max_value=None):
        super(IntParameter,self).__init__(required=required,default=default)
        self.min_value=min_value
        self.max_value=max_value
    def clean(self,value):
        try:
            i=int(value)
        except:
            raise InterfaceTypeError
        if (self.min_value is not None and i<self.min_value) or (self.max_value is not None and i>self.max_value):
            raise InterfaceTypeError
        return i
##
##
##
class ListParameter(Parameter):
    def clean(self,value):
        try:
            return list(value)
        except:
            raise InterfaceTypeError
##
##
##
class ListOfParameter(ListParameter):
    def __init__(self,element,required=True,default=None):
        super(ListOfParameter,self).__init__(required=required,default=default)
        self.element=element
        
    def clean(self,value):
        v=super(ListOfParameter,self).clean(value)
        return [self.element.clean(x) for x in v]
##
##
##
class StringListParameter(ListOfParameter):
    def __init__(self,required=True,default=None):
        super(StringListParameter,self).__init__(element=StringParameter(),required=required,default=default)
##
##
##
class DictParameter(Parameter):
    """
    >>> DictParameter(attrs={"i":IntParameter(),"s":StringParameter()}).clean({"i":10,"s":"ten"})
    {'i': 10, 's': 'ten'}
    >>> DictParameter(attrs={"i":IntParameter(),"s":StringParameter()}).clean({"i":"10","s":"ten"})
    {'i': 10, 's': 'ten'}
    """
    def __init__(self,required=True,default=None,attrs=None):
        super(DictParameter,self).__init__(required=required,default=default)
        self.attrs=attrs
    def clean(self,value):
        if type(value)!=types.DictType:
            raise InterfaceTypeError
        if not self.attrs:
            return value
        in_value=value.copy()
        out_value={}
        for a_name,attr in self.attrs.items():
            if a_name not in in_value and attr.required:
                raise InterfaceTypeError("Attribute '%s' required"%a_name)
            if a_name in in_value:
                try:
                    out_value[a_name]=attr.clean(in_value[a_name])
                except InterfaceTypeError:
                    raise InterfaceTypeError("Invalid value for '%s'"%a_name)
                del in_value[a_name]
        for k,v in in_value.items():
            out_value[k]=v
        return out_value
##
##
##
class IPParameter(StringParameter):
    """
    >>> IPParameter().clean("192.168.0.1")
    '192.168.0.1'
    >>> IPParameter().clean("192.168.0.256")
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    """
    def clean(self,value):
        v=super(IPParameter,self).clean(value)
        X=v.split(".")
        if len(X)!=4:
            raise InterfaceTypeError
        try:
            if len([x for x in X if 0<=int(x)<=255])!=4:
                raise InterfaceTypeError
        except:
            raise InterfaceTypeError
        return v
##
##
##
class VLANIDParameter(IntParameter):
    """
    >>> VLANIDParameter().clean(10)
    10
    >>> VLANIDParameter().clean(5000)
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    >>> VLANIDParameter().clean(0)
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    """
    def __init__(self,required=True,default=None):
        super(VLANIDParameter,self).__init__(required=required,default=default,min_value=1,max_value=4095)
##
##
##
rx_mac_address_cisco=re.compile("^[0-9A-F]{4}\.[0-9A-F]{4}\.[0-9A-F]{4}$")
rx_mac_address_sixblock=re.compile("^([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2})$")
class MACAddressParameter(StringParameter):
    """
    >>> MACAddressParameter().clean("1234.5678.9ABC")
    '12:34:56:78:9A:BC'
    >>> MACAddressParameter().clean("1234.5678.9abc")
    '12:34:56:78:9A:BC'
    >>> MACAddressParameter().clean("1234.5678.9abc.def0")
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    >>> MACAddressParameter().clean("12:34:56:78:9A:BC")
    '12:34:56:78:9A:BC'
    >>> MACAddressParameter().clean("12-34-56-78-9A-BC")
    '12:34:56:78:9A:BC'
    >>> MACAddressParameter().clean("0:13:46:50:87:5")
    '00:13:46:50:87:05'
    >>> MACAddressParameter().clean("12-34-56-78-9A-BC-DE")
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    >>> MACAddressParameter().clean("AB-CD-EF-GH-HJ-KL")
    Traceback (most recent call last):
        ...
    InterfaceTypeError
    """
    def clean(self,value):
        value=super(MACAddressParameter,self).clean(value)
        value=value.upper()
        match=rx_mac_address_cisco.match(value)
        if match:
            value=value.replace(".","")
        else:
            value=value.replace("-",":")
            match=rx_mac_address_sixblock.match(value)
            if not match:
                raise InterfaceTypeError
            value=""
            for i in range(1,7):
                v=match.group(i)
                if len(v)==1:
                    v="0"+v
                value+=v
        return "%s:%s:%s:%s:%s:%s"%(value[:2],value[2:4],value[4:6],value[6:8],value[8:10],value[10:])
##
##
##
class Interface(object):
    def clean(self,**kwargs):
        in_kwargs=kwargs.copy()
        out_kwargs={}
        for n,p in [(n,p) for n,p in self.__class__.__dict__.items() if issubclass(p.__class__,Parameter)]:
            if n=="returns":
                continue
            if n not in in_kwargs and p.required:
                raise InterfaceTypeError("Parameter '%s' required"%n)
            if n in in_kwargs:
                try:
                    out_kwargs[n]=p.clean(in_kwargs[n])
                except InterfaceTypeError:
                    raise InterfaceTypeError("Invalid value for '%s'"%n)
                del in_kwargs[n]
        # Copy other parameters
        for k,v in in_kwargs.items():
            out_kwargs[k]=v
        return out_kwargs
    def clean_result(self,result):
        try:
            rp=self.returns
        except AttributeError:
            return result # No return result restriction
        return rp.clean(result)
    def requires_input(self):
        for n,p in [(n,p) for n,p in self.__class__.__dict__.items() if issubclass(p.__class__,Parameter)]:
            if n=="returns":
                continue
            return True
        return False
    def get_form(self,data=None):
        def get_clean_field_wrapper(form,name,param):
            def clean_field_wrapper(form,name,param):
                data=form.cleaned_data[name]
                if not param.required and not data:
                    return data
                try:
                    return param.clean(data)
                except InterfaceTypeError:
                    raise forms.ValidationError("Invalid value")
            return lambda: clean_field_wrapper(form,name,param)
        f=forms.Form(data)
        for n,p in [(n,p) for n,p in self.__class__.__dict__.items() if issubclass(p.__class__,Parameter)]:
            if n=="returns":
                continue
            f.fields[n]=p.get_form_field()
            setattr(f,"clean_%s"%n,get_clean_field_wrapper(f,n,p))
        return f
##
## Module Test
##
if __name__ == "__main__":
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.dns.models import *

class Migration:
    
    def forwards(self):
        # Model 'DNSServer'
        db.create_table('dns_dnsserver', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True)),
            ('description', models.CharField("Description",max_length=128,blank=True,null=True)),
            ('location', models.CharField("Location",max_length=128,blank=True,null=True))
        ))
        
        # M2M field 'DNSZoneProfile.ns_servers'
        db.create_table('dns_dnszoneprofile_ns_servers', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('dnszoneprofile', models.ForeignKey(DNSZoneProfile, null=False)),
            ('dnsserver', models.ForeignKey(DNSServer, null=False))
        ))
        
        db.send_create_signal('dns', ['DNSServer'])
    
    def backwards(self):
        db.delete_table("dns_dnszoneprofile_ns_servers")
        db.delete_table('dns_dnsserver')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

class Migration:
    
    def forwards(self):
        AS=db.mock_model(model_name='AS', db_table='peer_as', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        if db.execute("SELECT COUNT(*) FROM peer_peeringpoint")[0][0]>0:
            db.add_column("peer_peeringpoint","local_as",models.ForeignKey(AS,verbose_name="Local AS",blank=True,null=True))
            as_id=db.execute("SELECT MIN(id) FROM peer_as")[0][0]
            db.execute("UPDATE peer_peeringpoint SET local_as_id=%s",[as_id])
            db.execute("ALTER TABLE peer_peeringpoint ALTER local_as_id SET NOT NULL")
        else:
            db.add_column("peer_peeringpoint","local_as",models.ForeignKey(AS,verbose_name="Local AS"))
        db.add_column("peer_peer","masklen",models.PositiveIntegerField("Masklen",default=30))
    
    def backwards(self):
        db.delete_column("peer_peeringpoint","local_as_id")
        db.delete_column("peer_peer","masklen")
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from base import *

class IGetVersion(Interface):
    returns=DictParameter(attrs={
                                "vendor"  : StringParameter(),
                                "platform": StringParameter(),
                                "version" : StringParameter(),
                                "image"   : StringParameter(required=False)
                                })



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVersion
import re

rx_sys=re.compile(r"System description:\s+.*?(?P<platform>ES.+?)$",re.MULTILINE|re.DOTALL)
rx_ver=re.compile(r"Operation code version\s+(?P<version>\S+)",re.MULTILINE|re.DOTALL)

class Script(noc.sa.script.Script):
    name="EdgeCore.ES35xx.get_version"
    implements=[IGetVersion]
    def execute(self):
        v=self.cli("show system")
        match_sys=rx_sys.search(v)
        v=self.cli("show version")
        match_ver=rx_ver.search(v.replace(":",""))
        return {
            "vendor"    : "EdgeCore",
            "platform"  : match_sys.group("platform"),
            "version"   : match_ver.group("version"),
        }



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.Report):
    name="cm.stale_configs"
    title="Stale Configs (2 days)"
    requires_cursor=True
    columns=[Column("Config"),Column("Last Pull")]
    
    def get_queryset(self):
        return self.execute("""
            SELECT repo_path,last_pull
            FROM cm_config WHERE last_pull IS NULL OR last_pull<('now'::timestamp-'2 days'::interval)
            ORDER BY 2
            """)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'EventPriority'
        db.create_table('fm_eventpriority', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('priority', models.IntegerField("Priority")),
            ('description', models.TextField("Description",blank=True,null=True))
        ))
        # Model 'EventCategory'
        db.create_table('fm_eventcategory', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True)),
            ('description', models.TextField("Description",blank=True,null=True))
        ))
        # Mock Models
        EventPriority = db.mock_model(model_name='EventPriority', db_table='fm_eventpriority', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        EventCategory = db.mock_model(model_name='EventCategory', db_table='fm_eventcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventClass'
        db.create_table('fm_eventclass', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64)),
            ('category', models.ForeignKey(EventCategory,verbose_name="Event Category")),
            ('default_priority', models.ForeignKey(EventPriority,verbose_name="Default Priority")),
            ('variables', models.CharField("Variables",max_length=128,blank=True,null=True)),
            ('subject_template', models.CharField("Subject Template",max_length=128)),
            ('body_template', models.TextField("Body Template")),
            ('last_modified', models.DateTimeField("last_modified",auto_now=True))
        ))
        # Mock Models
        EventClass = db.mock_model(model_name='EventClass', db_table='fm_eventclass', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventClassificationRule'
        db.create_table('fm_eventclassificationrule', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('event_class', models.ForeignKey(EventClass,verbose_name="Event Class")),
            ('name', models.CharField("Name",max_length=64)),
            ('preference', models.IntegerField("Preference",1000))
        ))
        
        # Mock Models
        EventClassificationRule = db.mock_model(model_name='EventClassificationRule', db_table='fm_eventclassificationrule', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventClassificationRE'
        db.create_table('fm_eventclassificationre', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('rule', models.ForeignKey(EventClassificationRule,verbose_name="Event Classification Rule")),
            ('left_re', models.CharField("Left RE",max_length=256)),
            ('right_re', models.CharField("Right RE",max_length=256))
        ))
        
        # Mock Models
        ManagedObject = db.mock_model(model_name='ManagedObject', db_table='sa_managedobject', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        Event = db.mock_model(model_name='Event', db_table='fm_event', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'Event'
        db.create_table('fm_event', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('timestamp', models.DateTimeField("Timestamp")),
            ('managed_object', models.ForeignKey(ManagedObject,verbose_name="Managed Object")),
            ('event_priority', models.ForeignKey(EventPriority,verbose_name="Priority")),
            ('event_category', models.ForeignKey(EventCategory,verbose_name="Event Class")),
            ('event_class', models.ForeignKey(EventClass,verbose_name="Event Class")),
            ('parent', models.ForeignKey(Event,verbose_name="Parent",blank=True,null=True)),
            ('subject', models.CharField("Subject",max_length=256,null=True,blank=True)),
            ('body', models.TextField("Body",null=True,blank=True))
        ))
        
        # Mock Models
        Event = db.mock_model(model_name='Event', db_table='fm_event', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'EventData'
        db.create_table('fm_eventdata', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('event', models.ForeignKey(Event,verbose_name=Event)),
            ('key', models.CharField("Key",max_length=64)),
            ('value', models.TextField("Value",blank=True,null=True)),
            ('type', models.CharField("Type",max_length=1,choices=[(">","Received"),("V","Variable"),("R","Resolved")],default=">"))
        ))
        db.create_index('fm_eventdata', ['event_id','key','type'], unique=True, db_tablespace='')
        
        db.send_create_signal('fm', ['EventPriority','EventCategory','EventClass','EventClassificationRule','EventClassificationRE',
            'Event','EventData'])
    
    def backwards(self):
        db.delete_table('fm_eventdata')
        db.delete_table('fm_event')
        db.delete_table('fm_eventclassificationre')
        db.delete_table('fm_eventclassificationrule')
        db.delete_table('fm_eventclass')
        db.delete_table('fm_eventcategory')
        db.delete_table('fm_eventpriority')



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Configuration Changes events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var

##
## Equipment configuraion changed
##
class ConfigChanged(EventClass):
    name     = "Config Changed"
    category = "NETWORK"
    priority = "INFO"
    subject_template="Configuration changed"
    body_template="""Equipment configuration changed"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger="on_config_changed"
    class Vars:
        user=Var(required=False)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco IOS specific process rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule
from noc.fm.rules.classes.process import *
##
## Cisco.IOS Process Traceback SYSLOG
##
class Cisco_IOS_Process_Traceback_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS Process Traceback SYSLOG"
    event_class=ProcessTraceback
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"Process= \"(?P<process>[^\"]+)\", .*, pid= (?P<pid>\d+),  -Traceback="),
    ]


# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.Report):
    name="fm.mibs"
    title="Loaded MIBs"
    requires_cursor=True
    columns=[
        Column("MIB"),
        Column("Last Updated"),
        Column("Uploaded"),
        Column("Count")]
    
    def get_queryset(self):
        return self.execute("""
            SELECT m.name,m.last_updated,m.uploaded,COUNT(*)
            FROM fm_mib m JOIN fm_mibdata d ON (d.mib_id=m.id)
            GROUP BY 1,2,3
            ORDER BY 1 DESC""")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import re,os,time,md5,subprocess
from django.db import models
from noc.settings import config
from noc.ip.models import IPv4Address
from noc.lib.validators import is_ipv4
from noc.lib.fileutils import is_differ,rewrite_when_differ,safe_rewrite
from noc.dns.generators import generator_registry
from noc.lib.rpsl import rpsl_format
from noc.main.menu import Menu

##
## register all generator classes
##
generator_registry.register_all()
##
## DNS Server
##
class DNSServer(models.Model):
    class Meta:
        verbose_name="DNS Server"
        verbose_name_plural="DNS Servers"
    name=models.CharField("Name",max_length=64,unique=True)
    generator_name=models.CharField("Generator",max_length=32,choices=generator_registry.choices)
    ip=models.IPAddressField("IP",null=True,blank=True)
    description=models.CharField("Description",max_length=128,blank=True,null=True)
    location=models.CharField("Location",max_length=128,blank=True,null=True)
    provisioning=models.CharField("Provisioning",max_length=128,blank=True,null=True,
        help_text="Script for zone provisioning")
    def __unicode__(self):
        if self.location:
            return "%s (%s)"%(self.name,self.location)
        else:
            return self.name
    def provision_zones(self):
        if self.provisioning:
            env=os.environ.copy()
            env["RSYNC_RSH"]=config.get("path","ssh")
            cmd=self.provisioning%{
                "rsync": config.get("path","rsync"),
                "ns"   : self.name,
                "ip"   : self.ip,
            }
            retcode=subprocess.call(cmd,shell=True,cwd=os.path.join(config.get("cm","repo"),"dns"),env=env)
            return retcode==0
    def _generator_class(self):
        return generator_registry[self.generator_name]
    generator_class=property(_generator_class)
##
##
##
class DNSZoneProfile(models.Model):
    class Meta:
        verbose_name="DNS Zone Profile"
        verbose_name_plural="DNS Zone Profiles"
    name=models.CharField("Name",max_length=32,unique=True)
    masters=models.ManyToManyField(DNSServer,verbose_name="Masters",related_name="masters",blank=True)
    slaves=models.ManyToManyField(DNSServer,verbose_name="Slaves",related_name="slaves",blank=True)
    zone_soa=models.CharField("SOA",max_length=64)
    zone_contact=models.CharField("Contact",max_length=64)
    zone_refresh=models.IntegerField("Refresh",default=3600)
    zone_retry=models.IntegerField("Retry",default=900)
    zone_expire=models.IntegerField("Expire",default=86400)
    zone_ttl=models.IntegerField("TTL",default=3600)

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    def _authoritative_servers(self):
        return list(self.masters.all())+list(self.slaves.all())
    authoritative_servers=property(_authoritative_servers)

##
## Managers for DNSZone
##
class ForwardZoneManager(models.Manager):
    def get_query_set(self):
        return super(ReverseZoneManager,self).get_query_set().exclude(name__endswith=".in-addr.arpa")
        
class ReverseZoneManager(models.Manager):
    def get_query_set(self):
        return super(ReverseZoneManager,self).get_query_set().filter(name__endswith=".in-addr.arpa")
##
##
rx_rzone=re.compile(r"^(\d+)\.(\d+)\.(\d+)\.in-addr.arpa$")
class DNSZone(models.Model):
    class Meta:
        verbose_name="DNS Zone"
        verbose_name_plural="DNS Zones"
        ordering=["name"]
    name=models.CharField("Domain",max_length=64,unique=True)
    description=models.CharField("Description",null=True,blank=True,max_length=64)
    is_auto_generated=models.BooleanField("Auto generated?")
    serial=models.CharField("Serial",max_length=10,default="0000000000")
    profile=models.ForeignKey(DNSZoneProfile,verbose_name="Profile")
    
    # Managers
    objects=models.Manager()
    forward_zones=ForwardZoneManager()
    reverse_zones=ReverseZoneManager()
    def __str__(self):
        return self.name
    def __unicode__(self):
        return self.name
    def _type(self):
        if self.name.endswith(".in-addr.arpa"):
            return "R"
        else:
            return "F"
    type=property(_type)
    def _reverse_prefix(self):
        match=rx_rzone.match(self.name)
        if match:
            return "%s.%s.%s.0/24"%(match.group(3),match.group(2),match.group(1))
    reverse_prefix=property(_reverse_prefix)
    def _next_serial(self):
        T=time.gmtime()
        p="%04d%02d%02d"%(T[0],T[1],T[2])
        sn=int(self.serial[-2:])
        if self.serial.startswith(p):
            return p+"%02d"%(sn+1)
        return p+"00"
    next_serial=property(_next_serial)
    
    def _records(self):
        from django.db import connection
        c=connection.cursor()
        if self.type=="F":
            c.execute("SELECT hostname(fqdn),ip FROM %s WHERE domainname(fqdn)=%%s ORDER BY ip"%IPv4Address._meta.db_table, [self.name])
            records=[[r[0],"IN  A",r[1]] for r in c.fetchall()]
        elif self.type=="R":
            c.execute("SELECT ip,fqdn FROM %s WHERE ip::cidr << %%s ORDER BY ip"%IPv4Address._meta.db_table,[self.reverse_prefix])
            records=[[r[0].split(".")[3],"PTR",r[1]+"."] for r in c.fetchall()]
        else:
            raise Exception,"Invalid zone type"
        # Add records from DNSZoneRecord
        zonerecords=self.dnszonerecord_set.all()
        if self.type=="R":
            # Subnet delegation macro
            delegations={}
            for d in [r for r in zonerecords if "NS" in r.type.type and "/" in r.left]:
                r=d.right
                l=d.left
                if l in delegations:
                    delegations[l].append(r)
                else:
                    delegations[l]=[r]
            for d,nses in delegations.items():
                try:
                    net,mask=[int(x) for x in l.split("/")]
                    if net<0 or net>255 or mask<=24 or mask>32:
                        raise Exception,"Invalid record"
                except:
                    records+=[[";; Invalid record: %s"%d,"IN NS","error"]]
                    continue
                for ns in nses:
                    records+=[[d,"IN NS",ns]]
                m=mask-24
                bitmask=((1<<m)-1)<<(8-m)
                if net&bitmask != net:
                    records+=[[";; Invalid network: %s"%d,"CNAME",d]]
                    continue
                for i in range(net,net+(1<<(8-m))):
                    records+=[["%d"%i,"CNAME","%d.%s"%(i,d)]]
            # Other records
            records+=[[x.left,x.type.type,x.right] for x in zonerecords\
                if ("NS" in x.type.type and "/" not in x.left) or "NS" not in x.type.type]
        else:
            records+=[[x.left,x.type.type,x.right] for x in zonerecords]
        # Add NS records if nesessary
        l=len(self.name)
        for z in self.children:
            for ns in z.ns_list:
                records+=[[z.name[:-l-1],"IN NS",ns]]
        records.sort(lambda x,y:cmp(x[0],y[0]))
        return records
    records=property(_records)
    
    def zonedata(self,ns):
        return ns.generator_class().get_zone(self)
    
    def _distribution_list(self):
        return self.profile.masters.filter(provisioning__isnull=False)
    distribution_list=property(_distribution_list)
    
    def distribution(self):
        return ", ".join(["<A HREF='/dns/%s/zone/%s/'>%s</A>"%(self.name,n.id,n.name) for n in self.distribution_list])
    distribution.short_description="Distribution"
    distribution.allow_tags=True

    def _children(self):
        l=len(self.name)
        return [z for z in DNSZone.objects.filter(name__iendswith="."+self.name) if "." not in z.name[:-l-1]]
    children=property(_children)
    
    def _ns_list(self):
        nses=[]
        for ns in self.profile.authoritative_servers:
            n=ns.name.strip()
            if not is_ipv4(n) and not n.endswith("."):
                n+="."
            nses.append(n)
        nses.sort()
        return nses
    ns_list=property(_ns_list)
    
    def _rpsl(self):
        if self.type!="R":
            return ""
        # Do not generate RPSL for private reverse zones
        if self.name.endswith(".10.in-addr.arpa"):
            return ""
        n1,n2,n=self.name.split(".",2)
        if n>="16.172.in-addr.arpa" and n<="31.172.in-addr.arpa":
            return ""
        n1,n=self.name.split(".",1)
        if n=="168.192.in-addr.arpa":
            return ""
        s=["domain: %s"%self.name]+["nserver: %s"%ns for ns in self.ns_list]
        return rpsl_format("\n".join(s))
    rpsl=property(_rpsl)
    
    def rpsl_link(self):
        return "<A HREF='/dns/%s/zone/rpsl/'>RPSL</A>"%self.name
    rpsl_link.short_description="RPSL"
    rpsl_link.allow_tags=True
##
##
##
class DNSZoneRecordType(models.Model):
    class Meta:
        verbose_name="DNS Zone Record Type"
        verbose_name_plural="DNS Zone Record Types"
        ordering=["type"]
    type=models.CharField("Type",max_length=16,unique=True)
    is_visible=models.BooleanField("Is Visible?",default=True)
    validation=models.CharField("Validation",max_length=256,blank=True,null=True,
        help_text="Regular expression to validate record. Following macros can be used: OCTET, IPv4, FQDN")
    def __str__(self):
        return self.type
    def __unicode__(self):
        return unicode(self.type)
    @classmethod
    def interpolate_re(self,rx):
        for m,s in [
            ("OCTET",r"(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)"),
            ("IPv4", r"(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)"),
            ("FQDN", r"([a-z0-9\-]+\.?)*"),]:
            rx=rx.replace(m,s)
        return "^%s$"%rx
    def is_valid(self,value):
        if self.validation:
            rx=DNSZoneRecordType.interpolate_re(self.validation)
            return re.match(rx,value) is not None
        else:
            return True
    def save(self):
        if self.validation:
            try:
                rx=DNSZoneRecordType.interpolate_re(self.validation)
            except:
                raise Exception("Invalid regular expression: %s"%rx)
            try:
                re.compile(rx)
            except:
                raise Exception("Invalid regular expression: %s"%rx)
        super(DNSZoneRecordType,self).save()
##
##
##
class DNSZoneRecord(models.Model):
    class Meta: pass
    zone=models.ForeignKey(DNSZone,verbose_name="Zone") # ,edit_inline=models.TABULAR,num_extra_on_change=5)
    left=models.CharField("Left",max_length=32,blank=True,null=True)
    type=models.ForeignKey(DNSZoneRecordType,verbose_name="Type")
    right=models.CharField("Right",max_length=64) #,core=True)
    def __str__(self):
        return "%s %s"%(self.zone.name," ".join([x for x in [self.left,self.type.type,self.right] if x is not None]))
    def __unicode__(self):
        return unicode(str(self))
##
## Application Menu
##
class AppMenu(Menu):
    app="dns"
    title="DNS"
    items=[
        ("Zones", "/admin/dns/dnszone/", "dns.change_dnszone"),
        ("Setup",[
            ("DNS Servers",      "/admin/dns/dnsserver/",         "dns.change_dnsserver"),
            ("Zone Profiles",    "/admin/dns/dnszoneprofile/",    "dns.change_dnszoneprofile"),
            ("Zone Record Types","/admin/dns/dnszonerecordtype/", "dns.change_dnszonerecordtype"),
        ])
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report
from noc.sa.profiles import profile_registry
from noc.sa.protocols.sae_pb2 import TELNET,SSH,HTTP

class Report(noc.main.report.Report):
    name="sa.supported_equipment"
    title="Supported Equipment"
    requires_cursor=False
    columns=[Column("Vendor"),Column("OS"),BooleanColumn("Telnet"),BooleanColumn("SSH"),BooleanColumn("HTTP"),
        BooleanColumn("Super command"),BooleanColumn("SNMP Config Trap"),BooleanColumn("Syslog Config"),BooleanColumn("LG Hilight"),
        Column("Scripts")]
    
    def get_queryset(self):
        def get_profile_scripts(p):
            print p,p.scripts
            return ", ".join(p.scripts.keys())
        r=[x for x in profile_registry.classes.items()]
        r.sort(lambda x,y:cmp(x[0],y[0]))
        return [x.split(".")\
            +[TELNET in c.supported_schemes,SSH in c.supported_schemes,HTTP in c.supported_schemes,c.command_super,c.oid_trap_config_changed,c.syslog_config_changed]\
            +[c.pattern_lg_as_path_list is not None or c.pattern_lg_best_path is not None,get_profile_scripts(c)] for x,c in r]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetDot11Associations
import re

rx_mac=re.compile("(?P<mac>[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2}:[0-9A-F]{2})")

class Script(noc.sa.script.Script):
    name="ZTE.ZXDSL531.get_dot11_associations"
    implements=[IGetDot11Associations]
    def execute(self):
        if self.access_profile.scheme==self.TELNET:
            v=self.cli("wlctl authe_sta_list")
        elif self.access_profile.scheme==self.HTTP:
            v=self.http.get("/wlclientview.cmd")
            v=self.strip_html_tags(v)
        else:
            raise Exception("Unsupported access scheme")
        r=[]
        for l in v.split("\n"):
            m=rx_mac.search(l)
            if m:
                r.append({"mac":m.group("mac")})
        return r



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVersion
import re

rx_ver=re.compile(r"^Board type: (?P<platform>.+), firmware version (?P<version>\S+)",re.MULTILINE|re.DOTALL)
rx_html_ver=re.compile(r"Version ID:\s+(?P<version>\S+)",re.MULTILINE|re.DOTALL)

class Script(noc.sa.script.Script):
    name="Audiocodes.Mediant2000.get_version"
    implements=[IGetVersion]
    def execute(self):
        if self.access_profile.scheme in [self.TELNET,self.SSH]:
            v=self.cli("show info")
            match=rx_ver.search(v)
            return {
                "vendor"    : "Audiocodes",
                "platform"  : match.group("platform"),
                "version"   : match.group("version"),
            }
        elif self.access_profile.scheme==self.HTTP:
            v=self.http.get("/SoftwareVersion")
            v=self.strip_html_tags(v)
            match=rx_html_ver.search(v)
            return {
                "vendor"    : "Audiocodes",
                "platform"  : "Mediant2000",
                "version"   : match.group("version"),
            }
        else:
            raise Exception("Unsupported access scheme")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.ip.models import *

class Migration:
    depends_on=(
        ("peer","0017_default_maintainer"),
    )
    def forwards(self):
        if db.execute("SELECT COUNT(*) FROM ip_vrfgroup")[0][0]==0:
            db.execute("INSERT INTO ip_vrfgroup(name) VALUES(%s)",["default"])
        if db.execute("SELECT COUNT(*) FROM peer_as WHERE asn=%s",[0])[0][0]==0:
            maintainer_id=db.execute("SELECT id FROM peer_maintainer LIMIT 1")[0][0]
            db.execute("INSERT INTO peer_as(asn,description,maintainer_id) VALUES(%s,%s,%s)",[0,"Default",maintainer_id])
        if db.execute("SELECT COUNT(*) FROM ip_vrf")[0][0]==0:
            vg_id=db.execute("SELECT id FROM ip_vrfgroup LIMIT 1")[0][0]
            db.execute("INSERT INTO ip_vrf(name,vrf_group_id,rd) VALUES(%s,%s,%s)",["default",vg_id,"0:0"])
            vrf_id=db.execute("SELECT id FROM ip_vrf LIMIT 1")[0][0]
            user_id=db.execute("SELECT id FROM auth_user LIMIT 1")[0][0]
            asn_id=db.execute("SELECT id FROM peer_as WHERE asn=%s",[0])[0][0]
            db.execute("INSERT INTO ip_ipv4block(prefix,description,vrf_id,asn_id,modified_by_id,last_modified) VALUES(%s,%s,%s,%s,%s,%s)",
                ["0.0.0.0/0","Root",vrf_id,asn_id,user_id,"now"]
            )
            
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import socket,struct
from django.db import models
from django.contrib.auth.models import User
from noc.lib.validators import check_rd,check_cidr,is_cidr
from noc.lib.tt import tt_url
from noc.peer.models import AS
from noc.lib.fields import CIDRField
from noc.lib.ip import int_to_address,bits_to_int,wildcard,broadcast
from noc.main.menu import Menu
##
##
##
class VRFGroup(models.Model):
    class Meta:
        verbose_name="VRF Group"
        verbose_name_plural="VRF Groups"
    name=models.CharField("VRF Group",unique=True,max_length=64)
    unique_addresses=models.BooleanField("Unique addresses in group")
    def __str__(self):
        return self.name
    def __unicode__(self):
        return unicode(self.name)
##
##
##
class VRF(models.Model):
    class Meta:
        verbose_name="VRF"
        verbose_name_plural="VRFs"
    name=models.CharField("VRF name",unique=True,max_length=64)
    vrf_group=models.ForeignKey(VRFGroup,verbose_name="VRF Group")
    rd=models.CharField("rd",unique=True,max_length=21) # validator_list=[check_rd],
    tt=models.IntegerField("TT",blank=True,null=True)
    def __str__(self):
        if self.rd=="0:0":
            return "global"
        return self.rd
    def __unicode__(self):
        if self.rd=="0:0":
            return u"global"
        return self.rd
        
    def prefix(self,top="0.0.0.0/0"):
        assert is_cidr(top)
        return IPv4Block.objects.get(prefix=top,vrf=self)
        
    def prefixes(self,top="0.0.0.0/0"):
        assert is_cidr(top)
        return self.prefix(top).children
        
    def all_prefixes(self,top="0.0.0.0/0"):
        assert is_cidr(top)
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT id FROM %s WHERE vrf_id=%d and prefix<<='%s' order by prefix"%(IPv4Block._meta.db_table,self.id,top))
        return [IPv4Block.objects.get(id=x[0]) for x in c.fetchall()]
        
    def all_addresses(self,top="0.0.0.0/0"):
        assert is_cidr(top)
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT id FROM %s WHERE vrf_id=%d and ip<<='%s' order by ip"%(IPv4Address._meta.db_table,self.id,top))
        return [IPv4Address.objects.get(id=x[0]) for x in c.fetchall()]
##
##
##
class IPv4BlockAccess(models.Model):
    class Meta:
        verbose_name="IPv4 Block Access"
        verbose_name_plural="IPv4 Block Access"
        unique_together=[("user","vrf","prefix")]
    user=models.ForeignKey(User,verbose_name="User")
    vrf=models.ForeignKey(VRF,verbose_name="VRF")
    prefix=CIDRField("prefix")
    tt=models.IntegerField("TT",blank=True,null=True)
    def __str__(self):
        return "%s: %s(%s)"%(self.user,self.prefix,self.vrf)
    @classmethod
    def check_write_access(self,user,vrf,prefix):
        assert is_cidr(prefix)
        from django.db import connection
        if user.is_superuser:
            return True
        c=connection.cursor()
        c.execute("SELECT COUNT(*) FROM %s WHERE prefix >>= '%s' AND vrf_id=%d AND user_id=%d"%(IPv4BlockAccess._meta.db_table,str(prefix),vrf.id,user.id))
        return c.fetchall()[0][0]>0
##
##
##
class IPv4Block(models.Model):
    class Meta:
        verbose_name="IPv4 Block"
        verbose_name_plural="IPv4 Blocks"
        unique_together=[("prefix","vrf")]
        ordering=["prefix"]
    description=models.CharField("Description",max_length=64)
    prefix=CIDRField("prefix")
    vrf=models.ForeignKey(VRF)
    asn=models.ForeignKey(AS)
    modified_by=models.ForeignKey(User,verbose_name="User")
    last_modified=models.DateTimeField("Last modified",auto_now=True,auto_now_add=True)
    tt=models.IntegerField("TT",blank=True,null=True)
    #allocated_till
    def __str__(self):
        return str(self.prefix)
    def __unicode__(self):
        return unicode(self.prefix)
    def _parent(self):
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT id FROM %s WHERE prefix >> '%s' AND vrf_id=%d ORDER BY masklen(prefix) DESC LIMIT 1"%(IPv4Block._meta.db_table,
                    self.prefix,self.vrf.id))
        r=c.fetchall()
        if len(r)==0:
            return
        return IPv4Block.objects.get(id=r[0][0])
    parent=property(_parent)

    def _parents(self):
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT id FROM %s WHERE prefix >> '%s' AND vrf_id=%d ORDER BY masklen(prefix)"%(IPv4Block._meta.db_table,
                    self.prefix,self.vrf.id))
        return [IPv4Block.objects.get(id=i[0]) for i in c.fetchall()]
    parents=property(_parents)

    def _depth(self):
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT COUNT(*) FROM %s WHERE prefix >> '%s' AND vrf_id=%d"%(IPv4Block._meta.db_table,self.prefix,self.vrf.id))
        return c.fetchone()[0]
    depth=property(_depth)

    def _children(self):
        from django.db import connection
        c=connection.cursor()
        if self.vrf.vrf_group.unique_addresses:
            vrfs="IN (%s)"%",".join([str(vrf.id) for vrf in self.vrf.vrf_group.vrf_set.all()])
            vg_id=self.vrf.vrf_group.id
            c.execute("SELECT b.id FROM %s b JOIN %s v ON (b.vrf_id=v.id) WHERE b.prefix << '%s' AND v.id %s AND ip_ipv4_block_depth_in_vrf_group(%d,b.prefix,'%s')=0 ORDER BY prefix"%\
                        (IPv4Block._meta.db_table,VRF._meta.db_table,self.prefix,vrfs,vg_id,self.prefix))
        else:
            vrfs="= %d"%self.vrf.id
            c.execute("SELECT b.id FROM %s b JOIN %s v ON (b.vrf_id=v.id) WHERE b.prefix << '%s' AND v.id %s AND ip_ipv4_block_depth(v.id,b.prefix,'%s')=0 ORDER BY prefix"%\
                        (IPv4Block._meta.db_table,VRF._meta.db_table,self.prefix,vrfs,self.prefix))
        return [IPv4Block.objects.get(id=i[0]) for i in c.fetchall()]

    children=property(_children)

    def _has_children(self):
        from django.db import connection
        c=connection.cursor()
        if self.vrf.vrf_group.unique_addresses:
            vrfs="IN (%s)"%",".join([str(vrf.id) for vrf in self.vrf.vrf_group.vrf_set.all()])
        else:
            vrfs="= %d"%self.vrf.id
        data=[]
        c.execute("SELECT COUNT(*) FROM %s b JOIN %s v ON (b.vrf_id=v.id) WHERE b.prefix << '%s' AND v.id %s AND ip_ipv4_block_depth(v.id,b.prefix,'%s')=0"%\
                    (IPv4Block._meta.db_table,VRF._meta.db_table,self.prefix,vrfs,self.prefix))
        return c.fetchall()[0][0]>0
    has_children=property(_has_children)

    def _addresses(self):
        if self.has_children:
            return None
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT id FROM %s WHERE vrf_id=%d AND ip << '%s' ORDER BY ip"%(IPv4Address._meta.db_table,self.vrf.id,self.prefix))
        return [IPv4Address.objects.get(id=i[0]) for i in c.fetchall()]
    addresses=property(_addresses)

    def _netmask_bits(self):
        return int(str(self.prefix).split("/")[1])
    netmask_bits=property(_netmask_bits)

    def _netmask(self):
        return int_to_address(bits_to_int(self.netmask_bits))
    netmask=property(_netmask)

    def _network(self):
        return str(self.prefix).split("/")[0]
    network=property(_network)

    def _wildcard(self): 
        return wildcard(self.prefix)
    wildcard=property(_wildcard)

    def _broadcast(self):
        return broadcast(self.prefix)
    broadcast=property(_broadcast)
    
    def _tt_url(self):
        return tt_url(self)
    tt_url=property(_tt_url)

##
##
##
class IPv4Address(models.Model):
    class Meta:
        unique_together=[("vrf","ip")]
        verbose_name="IPv4 Address"
        verbose_name_plural="IPv4 Addresses"
        ordering=["ip"]
    vrf=models.ForeignKey(VRF,verbose_name="VRF")
    fqdn=models.CharField("FQDN",max_length=64)
    ip=models.IPAddressField("IP")
    description=models.CharField("Description",blank=True,null=True,max_length=64)
    modified_by=models.ForeignKey(User,verbose_name="User")
    last_modified=models.DateTimeField("Last modified",auto_now=True,auto_now_add=True)
    tt=models.IntegerField("TT",blank=True,null=True)
    
    def __unicode__(self):
        return self.ip

    def __str__(self):
        return self.ip

    def _parent(self):
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT id FROM %s WHERE vrf_id=%d AND '%s' << prefix ORDER BY masklen(prefix) DESC LIMIT 1"%(IPv4Block._meta.db_table,self.vrf.id,str(self.ip)))
        return IPv4Block.objects.get(id=c.fetchall()[0][0])
    parent=property(_parent)
    
    def _tt_url(self):
        return tt_url(self)
    tt_url=property(_tt_url)
##
## Application Menu
##
class AppMenu(Menu):
    app="ip"
    title="Address Space Management"
    items=[
        ("Assigned addresses", "/ip/", "ip.change_ipv4block"),
        ("Setup",[
            ("VRF Groups",  "/admin/ip/vrfgroup/",        "ip.change_vrfgroup"),
            ("VRFs",        "/admin/ip/vrf/",             "ip.change_vrf"),
            ("Block Access","/admin/ip/ipv4blockaccess/", "ip.change_ipv4blockaccess"),
            ]
        )
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVlans

class Script(noc.sa.script.Script):
    name="Huawei.VRP.get_vlans"
    implements=[IGetVlans]
    def execute(self):
        self.cli("undo terminal logging")
        vlans=self.cli("display vlan").split("\n")
        vlans=vlans[1].replace("(default)","")
        return [{"vlan_id":int(x)} for x in self.expand_rangelist(vlans)]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Vendor: Cisco
## OS:     IOS
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.profiles
from noc.sa.protocols.sae_pb2 import TELNET,SSH

class Profile(noc.sa.profiles.Profile):
    name="Cisco.IOS"
    supported_schemes=[TELNET,SSH]
    pattern_more="^ --More--"
    pattern_unpriveleged_prompt=r"^\S+?>"
    command_super="enable"
    pattern_prompt=r"^\S+?#"
    pattern_lg_as_path_list=r"^(\s+\d+(?: \d+)*),"
    pattern_lg_best_path=r"(<A HREF.+?>.+?best)"
    requires_netmask_conversion=True
    convert_mac=noc.sa.profiles.Profile.convert_mac_to_cisco
    config_volatile=["^ntp clock-period .*?^"]
    oid_trap_config_changed="1.3.6.1.4.1.9.9.43.2"
    syslog_config_changed="%SYS-5-CONFIG_I: Configured from"
    
    def generate_prefix_list(self,name,pl,strict=True):
        p="ip prefix-list %s permit %%s"%name
        if not strict:
            p+=" le 32"
        return "no ip prefix-list %s\n"%name+"\n".join([p%x for x in pl])
    



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Data type validators
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import re
from django.forms import ValidationError
##
## Validators returning boolean
##

##
## Check value is valid integer
##
def is_int(v):
    try:
        v=int(v)
    except:
        return False
    return True
##
## Check value is valid 2-byte autonomous system number
##
def is_asn(v):
    try:
        v=int(v)
    except:
        return False
    return 0<=v<=65535
##
## Check value is valid IPv4 address
##
def is_ipv4(v):
    X=v.split(".")
    if len(X)!=4:
        return False
    try:
        return len([x for x in X if 0<=int(x)<=255])==4
    except:
        return False
##
## Check value is valid IPv4 prefix
##
def is_cidr(v):
    x=v.split("/")
    if len(x)!=2:
        return False
    if not is_ipv4(x[0]):
        return False
    try:
        y=int(x[1])
    except:
        return False
    return 0<=y<=32
##
## Check value is valid Route Distinguisher
##
def is_rd(v):
    x=v.split(":")
    if len(x)!=2:
        return False
    a,b=x
    try:
        b=int(b)
    except:
        return False
    if is_asn(a):
        return 0<=b<=16777215
    if is_asn(a):
        return 0<=b<=65535
    return False
##
## Check value is valuid AS-SET
##
rx_asset=re.compile(r"^AS-[A-Z0-9\-]+$")
def is_as_set(v):
    return rx_asset.match(v) is not None
##
## Check value is valid FQDN
##
rx_fqdn=re.compile(r"^([a-z0-9\-]+\.)+[a-z0-9\-]+$",re.IGNORECASE)
def is_fqdn(v):
    return rx_fqdn.match(v) is not None
##
## Check value is valid regular expression
##
def is_re(v):
    try:
        re.compile(v)
        return True
    except:
        return False
##
## Validators for forms
##
def generic_validator(value,check,error_msg):
    if not check(value):
        raise ValidationError(error_msg)

def check_asn(field_data,all_data):
    generic_validator(field_data,is_asn,"Invalid ASN")

def check_ipv4(field_data,all_data):
    generic_validator(field_data,is_ipv4,"Invalid IPv4")

def check_cidr(field_data,all_data):
    generic_validator(field_data,is_cidr,"Invalid CIDR")

def check_rd(field_data,all_data):
    generic_validator(field_data,is_rd,"Invalid RD")
    
def check_fqdn(field_data,all_data):
    generic_validator(field_data,is_fqdn,"Invalid FQDN")
    
def check_as_set(field_data,all_data):
    generic_validator(field_data,is_as_set,"Invalid AS-SET")

def check_re(field_data,all_data):
    generic_validator(field_data,is_re,"Invalid Regular Expression")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import os,tempfile,sha

def safe_rewrite(path,text):
    d=os.path.dirname(path)
    if not os.path.exists(d):
        os.makedirs(d)
    b=os.path.basename(path)
    h,p=tempfile.mkstemp(suffix=".tmp",prefix=b,dir=d)
    f=os.fdopen(h,"w")
    f.write(text)
    f.close()
    if os.path.exists(path):
        os.unlink(path)
    os.link(p,path)
    os.unlink(p)
##
## Check file content is differ from string
##
def is_differ(path,content):
    if os.path.isfile(path):
        f=open(path)
        cs1=sha.sha(f.read()).digest()
        f.close()
        cs2=sha.sha(content).digest()
        return cs1!=cs2
    else:
        return True
##
## Rewrites file when content is differ
## Returns boolean signalling wherher file was rewritten
##
def rewrite_when_differ(path,content):
    d=is_differ(path,content)
    if d:
        safe_rewrite(path,content)
    return d
##
## Read file and return file's content.
## Return None when file does not exists
##
def read_file(path):
    if os.path.exists(path):
        f=open(path,"r")
        data=f.read()
        f.close()
        return data
    else:
        return None
##
## Copy File
##
def copy_file(f,t):
    d=read_file(f)
    if d is None:
        d=""
    safe_rewrite(t,d)
##
## Create temporary file, write content and return path
##
def write_tempfile(text):
    h,p=tempfile.mkstemp()
    f=os.fdopen(h,"w")
    f.write(text)
    f.close()
    return p
##
## Temporary file context manager.
## Writes data to temporary file an returns path.
## Unlinks temporary file on exit
## USAGE:
##     with temporary_file("line1\nline2") as p:
##         subprocess.Popen(["wc","-l",p])
##
class temporary_file(object):
    def __init__(self,text):
        self.text=text
    def __enter__(self):
        self.p=write_tempfile(self.text)
        return self.p
    def __exit__(self, type, value, tb):
        os.unlink(self.p)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## URL Processing functions
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import re

rx_url=re.compile(r"^(?P<scheme>[^:]+)://(?P<user>[^:]+):(?P<password>[^@]+)@(?P<host>[^/]+)(?:(?P<port>\d+))?(?P<path>.*)$")

class InvalidURLException(Exception): pass

class URL(object):
    def __init__(self,url):
        self.url=url
        match=rx_url.match(self.url)
        if not match:
            raise InvalidURLException
        self.scheme=match.group("scheme")
        self.user=match.group("user")
        self.password=match.group("password")
        self.host=match.group("host")
        self.port=match.group("port")
        self.path=match.group("path")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.Report):
    name="fm.event_priority_events"
    title="Event Priority Summary"
    requires_cursor=True
    columns=[
        Column("Event Priority"),
        Column("Events")]
    
    def get_queryset(self):
        return self.execute("""
            SELECT ep.name,COUNT(*)
            FROM fm_eventpriority ep JOIN fm_event e ON (ep.id=e.event_priority_id)
            GROUP BY 1
            ORDER BY 2 DESC""")




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
from noc.ip.models import VRF
from lib.validators import is_cidr
from django import forms

class ReportForm(forms.Form):
    vrf=forms.ModelChoiceField(label="VRF",queryset=VRF.objects)
    prefix=forms.CharField(label="Prefix",initial="0.0.0.0/0")
    
    def clean_prefix(self):
        prefix=self.cleaned_data.get("prefix","").strip()
        if not is_cidr(prefix):
            raise forms.ValidationError("Invalid prefix")
        return prefix

class Report(noc.main.report.Report):
    name="ip.allocations"
    title="Allocated Blocks"
    form_class=ReportForm
    requires_cursor=True
    columns=[Column("Block"),Column("Description")]
    
    def get_queryset(self):
        vrf_id=self.form.cleaned_data["vrf"].id
        prefix=self.form.cleaned_data["prefix"]
        return self.execute("""
            SELECT prefix,description
            FROM ip_ipv4block b
            WHERE vrf_id=%s
                AND prefix<<%s::cidr
                AND (SELECT COUNT(*) FROM ip_ipv4block bb WHERE vrf_id=%s AND bb.prefix<<b.prefix)=0
            ORDER BY prefix
        """,[vrf_id,prefix,vrf_id])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## System event classes description
## System events are generated by NOC itself
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Unhandled python exception in daemon
##
class UnhandledException(EventClass):
    name="Unhandled Exception"
    category="SYSTEM"
    priority="CRITICAL"
    subject_template="Exception: {{component}}"
    body_template="""Unhandled exception at {{component}}
-----------
{{traceback}}
-----------
    """
    class Vars:
        source=Var(required=True)
        type=Var(required=True)
        component=Var(required=True)
        traceback=Var(required=True)
##
## Activator reveived message from unknown source address
##
class InvalidEventSource(EventClass):
    name="Invalid Event Source"
    category="SYSTEM"
    priority="WARNING"
    subject_template="Invalid event source: {{ip}}"
    body_template="""Invalid event source {{ip}} at activator {{activator}}"""
    class Vars:
        source=Var(required=True)
        type=Var(required=True)
        activator=Var(required=True)
        ip=Var(required=True)
##
## Periodic task terminated with failure
##
class PeriodicTaskFailed(EventClass):
    name="Periodic Task Failed"
    category="SYSTEM"
    priority="CRITICAL"
    subject_template="Periodic task failed: {{periodic}}"
    body_template="""Periodic task failed: {{periodic}}"""
    class Vars:
        source=Var(required=True)
        type=Var(required=True)
        periodic=Var(required=True)
##
## Managed Object Unreachable during ping probe
##
class ObjectUnreachable(EventClass):
    name="Object Unreachable"
    category="NETWORK"
    priority="MAJOR"
    subject_template="DOWN: {{ip}}"
    body_template="""Object unreachable: {{ip}}"""
    class Vars:
        source=Var(required=True)
        activator=Var(required=True)
        probe=Var(required=True)
        result=Var(required=True)
        ip=Var(required=True)
##
## Managed Reachable during ping probe
##
class ObjectReachable(EventClass):
    name="Object Reachable"
    category="NETWORK"
    priority="INFO"
    subject_template="UP: {{ip}}"
    body_template="""Object reachable: {{ip}}"""
    class Vars:
        source=Var(required=True)
        activator=Var(required=True)
        probe=Var(required=True)
        result=Var(required=True)
        ip=Var(required=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Small DSL for event class description
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Link UP
##
class LinkUp(EventClass):
    name="Link Up"
    category="NETWORK"
    priority="MAJOR"
    subject_template="Link Up: {{interface}}"
    body_template="""Link Up: {{interface}}"""
    class Vars:
        interface=Var(required=True)
##
## Link Down
##
class LinkDown(EventClass):
    name="Link Down"
    category="NETWORK"
    priority="MAJOR"
    subject_template="Link Down: {{interface}}"
    body_template="""Link Down: {{interface}}"""
    class Vars:
        interface=Var(required=True)




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from noc.peer.models import RIR,Person,Maintainer,AS,ASSet,PeeringPoint,PeerGroup,Peer,CommunityType,Community

class RIRAdmin(admin.ModelAdmin): pass

class PersonAdmin(admin.ModelAdmin):
    list_display=["nic_hdl","person","rir","rpsl_link"]
    list_filter=["rir"]
    search_fields=["nic_hdl","person"]

class MaintainerAdmin(admin.ModelAdmin):
    list_display=["maintainer","description","rir","rpsl_link"]
    list_filter=["rir"]

class ASAdmin(admin.ModelAdmin):
    list_display=["asn","description","maintainer","rpsl_link"]
    list_filter=["maintainer"]
    search_fields=["asn","description"]
    
class CommunityTypeAdmin(admin.ModelAdmin):
    list_display=["name"]
    
class CommunityAdmin(admin.ModelAdmin):
    list_display=["community","type","description"]
    list_filter=["type"]
    search_fields=["community","description"]
        
class ASSetAdmin(admin.ModelAdmin):
    list_display=["name","description","members","rpsl_link"]
    search_fields=["name","description","members"]

class PeeringPointAdmin(admin.ModelAdmin):
    list_display=["hostname","location","local_as","router_id","profile_name","communities","rpsl_link"]
    list_filter=["profile_name"]
    search_fields=["hostname","router_id"]
        
class PeerGroupAdmin(admin.ModelAdmin):
    list_display=["name","description","communities"]
        
class PeerAdmin(admin.ModelAdmin):
    list_display=["peering_point","local_asn","remote_asn","admin_import_filter","admin_export_filter","local_ip","masklen","remote_ip","admin_tt_url","description","communities"]
    search_fields=["remote_asn","description"]
    list_filter=["peering_point"]

admin.site.register(RIR,RIRAdmin)
admin.site.register(Person,PersonAdmin)
admin.site.register(Maintainer,MaintainerAdmin)
admin.site.register(AS,ASAdmin)
admin.site.register(CommunityType,CommunityTypeAdmin)
admin.site.register(Community,CommunityAdmin)
admin.site.register(ASSet,ASSetAdmin)
admin.site.register(PeeringPoint,PeeringPointAdmin)
admin.site.register(PeerGroup,PeerGroupAdmin)
admin.site.register(Peer,PeerAdmin)




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.add_column("fm_eventclass","trigger",models.CharField("Trigger",max_length=64,null=True,blank=True))
    
    def backwards(self):
        db.delete_column("fm_eventclass","trigger")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Fallback event classes description
## Fallback classes are based on event source type only (SNMP Trap, Syslog)
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var

##
## Fallback for all unprocessed SNMP traps
##
class UnhandledSNMPTrap(EventClass):
    name="SNMP Trap"
    subject_template="trap: {{oid}}"
    body_template="""Unhandled SNMP trap:
----------
{{oid}}
----------"""
    class Vars:
        oid=Var(required=True)

##
## Fallback for all unprocessed Syslog messages
##
class UnhandledSyslog(EventClass):
    name="SYSLOG"
    subject_template="syslog: {{message}}"
    body_template="""Unhandled syslog message:
----------
{{message}}
----------"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    class Vars:
        message=Var(required=True,repeat=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## SNMP Trap Collector
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from pyasn1.codec.ber import decoder
from pysnmp.proto import api
from noc.lib.nbsocket import ListenUDPSocket
from noc.sa.eventcollector import EventCollector
from noc.lib.pyquote import bin_quote
import time

##
## Body has following structure:
## {
## "source"   : "SNMP trap"
## "oid"      : <trap oid>
## "left oid" : <right value>
## }
##
class TrapCollector(ListenUDPSocket,EventCollector):
    def __init__(self,activator,address,port):
        self.info("Initializing")
        ListenUDPSocket.__init__(self,activator.factory,address,port)
        EventCollector.__init__(self,activator)
        
    def on_read(self,whole_msg,address,port):
        def oid_to_str(o):
            return ".".join([str(x) for x in o])
        def extract(val):
            def unchain(val): 
                c = []
                for i in range(len(val._componentValues)):
                    k=val._componentValues[i]
                    if k is not None:
                        if hasattr(k,"getComponentType"):
                            c.append(k.getComponentType().getNameByPosition(i))
                            c.extend(unchain(k))
                        elif hasattr(k,"_value"):
                            c.append(k._value)
                return c
                
            v=unchain(val)
            if len(v)==0:
                return ""
            v=v[-1]
            if type(v)==tuple:
                return oid_to_str(v)
            return bin_quote(str(v))

        if not self.check_source_address(address):
            return
        while whole_msg:
            msg_version = int(api.decodeMessageVersion(whole_msg))
            if api.protoModules.has_key(msg_version):
                p_mod = api.protoModules[msg_version]
            else:
                self.error('Unsupported SNMP version %s from %s'%(msg_version,address))
                return
            req_msg,whole_msg=decoder.decode(whole_msg,asn1Spec=p_mod.Message())
            req_pdu = p_mod.apiMessage.getPDU(req_msg)
            if req_pdu.isSameTypeWith(p_mod.TrapPDU()):
                body={"source":"SNMP Trap"}
                if msg_version==api.protoVersion1:
                    oid=oid_to_str(p_mod.apiTrapPDU.getEnterprise(req_pdu))
                    body["1.3.6.1.6.3.1.1.4.1.0"]=oid # snmpTrapOID.0
                    var_binds=p_mod.apiTrapPDU.getVarBindList(req_pdu)
                else:
                    var_binds=p_mod.apiPDU.getVarBindList(req_pdu)
                ts=int(time.time())
                for o,v in var_binds:
                    body[oid_to_str(o._value)]=extract(v)
                self.process_event(ts,address,body)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from django import forms
from noc.dns.models import DNSServer,DNSZoneProfile,DNSZone,DNSZoneRecordType,DNSZoneRecord

class DNSServerAdmin(admin.ModelAdmin):
    list_display=["name","generator_name","ip","location","description"]
    search_fields=["name","description","ip"]
    list_filter=["generator_name"]
    
class DNSZoneProfileAdmin(admin.ModelAdmin):
    pass

class DNSZoneRecordInlineForm(forms.ModelForm):
    class Meta:
        model=DNSZoneRecord
    def clean_right(self):
        s=[]
        if "left" in self.cleaned_data:
            s+=[self.cleaned_data["left"]]
        s+=[self.cleaned_data["type"].type]
        if "right" in self.cleaned_data:
            s+=[self.cleaned_data["right"]]
        s=" ".join(s)
        if not self.cleaned_data["type"].is_valid(s):
            raise forms.ValidationError("Invalid record")
        return self.cleaned_data["right"]
        
class DNSZoneRecordInline(admin.TabularInline):
    form=DNSZoneRecordInlineForm
    model=DNSZoneRecord
    extra=3
    
class DNSZoneAdmin(admin.ModelAdmin):
    inlines=[DNSZoneRecordInline]
    list_display=["name","description","is_auto_generated","serial","distribution","rpsl_link"]
    list_filter=["is_auto_generated"]
    search_fields=["name","description"]
    
class DNSZoneRecordTypeAdmin(admin.ModelAdmin):
    list_display=["type","is_visible","validation"]
    search_fields=["type"]
    list_filter=["is_visible"]

admin.site.register(DNSServer, DNSServerAdmin)
admin.site.register(DNSZoneProfile,DNSZoneProfileAdmin)
admin.site.register(DNSZone,DNSZoneAdmin)
admin.site.register(DNSZoneRecordType,DNSZoneRecordTypeAdmin)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetVersion
import re

rx_html_ver=re.compile(r"Firmware Version\s+\S+\s+(?P<version>\S+)")

class Script(noc.sa.script.Script):
    name="ZTE.ZXDSL531.get_version"
    implements=[IGetVersion]
    def execute(self):
        if self.access_profile.scheme==self.TELNET:
            v=self.cli("swversion show")
            platform,version=v.split()
        elif self.access_profile.scheme==self.HTTP:
            v=self.http.get("/info.html")
            v=self.strip_html_tags(v)
            match=rx_html_ver.search(v)
            version=match.group("version")
        else:
            raise Exception("Unsupported access scheme")
        return {
            "vendor"    : "ZTE",
            "platform"  : "ZXDSL531",
            "version"   : version,
        }



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report

class Report(noc.main.report.Report):
    name="sa.objects_by_profiles"
    title="Objects by Profiles"
    requires_cursor=True
    columns=[Column("Profile"),Column("Qty")]
    
    def get_queryset(self):
        return self.execute("SELECT profile_name,COUNT(*) FROM sa_managedobject GROUP BY 1 ORDER BY 2 DESC")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Process-related events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Process Traceback
##
class ProcessTraceback(EventClass):
    name     = "Process Traceback"
    category = "NETWORK"
    priority = "CRITICAL"
    subject_template="Process traceback: {{process}}"
    body_template="""Process crashed: {{process}}
PID: {{pid}}"""
    repeat_suppression=False
    repeat_suppression_interval=3600
    trigger=None
    class Vars:
        pid=Var(required=False,repeat=False)
        process=Var(required=False,repeat=False)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Runs PING probe of all hosts
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.periodic
from django.db.models import Q
import datetime,logging

class Task(noc.sa.periodic.Task):
    name="fm.ping_check"
    description=""
    
    def execute(self):
        from noc.sa.models import Activator
        
        for a in Activator.objects.filter(is_active=True):
            objects=[o.trap_source_ip for o in a.managedobject_set.filter(trap_source_ip__isnull=False)]
            if objects:
                self.sae.ping_check(a,objects)
        return True




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.shortcuts import get_object_or_404
from django.http import HttpResponseRedirect,HttpResponseForbidden
from noc.dns.models import DNSZone,DNSServer
from noc.lib.render import render,render_plain_text

def zone(request,zone,ns_id):
    z=get_object_or_404(DNSZone,name=zone)
    ns=get_object_or_404(DNSServer,id=int(ns_id))
    return render_plain_text(z.zonedata(ns))

def zone_rpsl(request,zone):
    z=get_object_or_404(DNSZone,name=zone)
    return render_plain_text(z.rpsl)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.core.management.base import BaseCommand
from noc.cm.models import Object


class Command(BaseCommand):
    help="Push objects"
    def handle(self, *args, **options):
        Object.global_push(args[0])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Syslog Collector
## (RFC3164)
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.nbsocket import ListenUDPSocket
from noc.sa.eventcollector import EventCollector
import time
##
## Body has following keys:
##  source, facility, severity, message
##
##
class SyslogCollector(ListenUDPSocket,EventCollector):
    def __init__(self,activator,address,port):
        self.info("Initializing")
        ListenUDPSocket.__init__(self,activator.factory,address,port)
        EventCollector.__init__(self,activator)
        
    def on_read(self,msg,address,port):
        self.debug(msg)
        if not self.check_source_address(address):
            return
        self.debug(">>>")
        # Parse priority
        priority=0
        if msg.startswith("<"):
            idx=msg.find(">")
            if idx==-1:
                return
            try:
                priority=int(msg[1:idx])
            except:
                pass
            msg=msg[idx+1:].strip()
        # Parse HEADER if exists
        ts=int(time.time())
        #
        body={
            "source"  : "syslog",
            "facility": priority>>3,
            "severity": priority&7,
            "message" : msg
        }
        self.process_event(ts,address,body)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

class Migration:
    
    def forwards(self):
        
        # Model 'LGQueryType'
        db.create_table('peer_lgquerytype', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True))
        ))
        
        # Mock Models
        PeeringPointType = db.mock_model(model_name='PeeringPointType', db_table='peer_peeringpointtype', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        LGQueryType = db.mock_model(model_name='LGQueryType', db_table='peer_lgquerytype', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'LGQueryCommand'
        db.create_table('peer_lgquerycommand', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('peering_point_type', models.ForeignKey(PeeringPointType,verbose_name="Peering Point Type")),
            ('query_type', models.ForeignKey(LGQueryType,verbose_name="LG Query Type")),
            ('command', models.CharField("Command",max_length=128))
        ))
        db.create_index('peer_lgquerycommand', ['peering_point_type_id','query_type_id'], unique=True, db_tablespace='')
        db.send_create_signal('peer', ['LGQueryType','LGQueryCommand'])
    
    def backwards(self):
        db.delete_table('peer_lgquerycommand')
        db.delete_table('peer_lgquerytype')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.dns.models import *

class Migration:
    
    def forwards(self):
        # Mock Models
        DNSZoneProfile = db.mock_model(model_name='DNSZoneProfile', db_table='dns_dnszoneprofile', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        DNSServer = db.mock_model(model_name='DNSServer', db_table='dns_dnsserver', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'DNSZoneProfile.masters'
        db.create_table('dns_dnszoneprofile_masters', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('dnszoneprofile', models.ForeignKey(DNSZoneProfile, null=False)),
            ('dnsserver', models.ForeignKey(DNSServer, null=False))
        )) 
        # Mock Models
        DNSZoneProfile = db.mock_model(model_name='DNSZoneProfile', db_table='dns_dnszoneprofile', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        DNSServer = db.mock_model(model_name='DNSServer', db_table='dns_dnsserver', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'DNSZoneProfile.slaves'
        db.create_table('dns_dnszoneprofile_slaves', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('dnszoneprofile', models.ForeignKey(DNSZoneProfile, null=False)),
            ('dnsserver', models.ForeignKey(DNSServer, null=False))
        ))
        db.execute("INSERT INTO dns_dnszoneprofile_masters(dnszoneprofile_id,dnsserver_id) SELECT dnszoneprofile_id,dnsserver_id FROM dns_dnszoneprofile_ns_servers")
        db.delete_table('dns_dnszoneprofile_ns_servers')
        
        db.send_create_signal('dns', ['DNSZoneProfile'])
    
    def backwards(self):
        # Mock Models
        DNSZoneProfile = db.mock_model(model_name='DNSZoneProfile', db_table='dns_dnszoneprofile', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        DNSServer = db.mock_model(model_name='DNSServer', db_table='dns_dnsserver', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # M2M field 'DNSZoneProfile.slaves'
        db.create_table('dns_dnszoneprofile_ns_servers', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('dnszoneprofile', models.ForeignKey(DNSZoneProfile, null=False)),
            ('dnsserver', models.ForeignKey(DNSServer, null=False))
        ))
        db.execute("INSERT INTO dns_dnszoneprofile_ns_servers(dnszoneprofile_id,dnsserver_id) SELECT dnszoneprofile_id,dnsserver_id FROM dns_dnszoneprofile_masters UNION SELECT dnszoneprofile_id,dnsserver_id FROM dns_dnszoneprofile_slaves")
        
        db.delete_table('dns_dnszoneprofile_masters')
        db.delete_table('dns_dnszoneprofile_slaves')



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Authentication Events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## Authentication Failed
##
class AuthenticationFailure(EventClass):
    name="Authentication Failure"
    category="SECURITY"
    priority="MAJOR"
    subject_template="Authentication failure from {{ip}}"
    body_template="""Authentication failure:
USER: {{user}}
IP: {{ip}}
"""
    class Vars:
        ip=Var(required=False)
        user=Var(required=False)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## IPsec Events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## IPsec Phase1 Tunnel Start
##
class IPsecPhase1TunnelStart(EventClass):
    name     = "IPsec Phase1 Tunnel Start"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="IPsec Phase1 Tunnel Start: {{local_ip}} - {{remote_ip}}"
    body_template="""IPsec Phase1 Tunnel Start:
{{local_ip}} - {{remote_ip}}"""
    class Vars:
        local_ip=Var(required=True)
        remote_ip=Var(required=True)
##
## IPsec Phase1 Tunnel Stop
##
class IPsecPhase1TunnelStop(EventClass):
    name     = "IPsec Phase1 Tunnel Stop"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="IPsec Phase1 Tunnel Stop: {{local_ip}} - {{remote_ip}}"
    body_template="""IPsec Phase1 Tunnel Stop:
{{local_ip}} - {{remote_ip}}"""
    class Vars:
        local_ip=Var(required=True)
        remote_ip=Var(required=True)
#
#class IPsecPhase1NoSA(EventClass):
#    pass
##
## IPsec Phase2 Tunnel Start
##
class IPsecPhase2TunnelStart(EventClass):
    name     = "IPsec Phase2 Tunnel Start"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="IPsec Phase2 Tunnel Start: {{local_ip}} - {{remote_ip}}"
    body_template="""IPsec Phase2 Tunnel Start:
{{local_ip}} - {{remote_ip}}"""
    class Vars:
        local_ip=Var(required=True)
        remote_ip=Var(required=True)
##
## IPsec Phase2 Tunnel Stop
##
class IPsecPhase2TunnelStop(EventClass):
    name     = "IPsec Phase2 Tunnel Stop"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="IPsec Phase2 Tunnel Stop: {{local_ip}} - {{remote_ip}}"
    body_template="""IPsec Phase2 Tunnel Stop:
{{local_ip}} - {{remote_ip}}"""
    class Vars:
        local_ip=Var(required=True)
        remote_ip=Var(required=True)
#
#class IPsecPhase2NoSA(EventClass):
#    pass
#
##
## IPsec Invalid SPI
##
class IPsecInvalidSPI(EventClass):
    name     = "IPsec Invalid SPI"
    category = "NETWORK"
    priority = "MAJOR"
    subject_template="IPsec Invalid packet SPI: {{src_ip}} - {{dst_ip}}"
    body_template="""IPsec Invalid packet SPI:
{{src_ip}} - {{dst_ip}}"""
    class Vars:
        src_ip=Var(required=True)
        dst_ip=Var(required=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## ECMA-48 control sequences processing
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import re
## Convert ECMA Notation to byte form
def c(x,y):
    """
    Convert ECMA-48 character notation to 8-bit form
    >>> c(0,0)
    0
    >>> c(1,11)
    27
    >>> c(15,15)
    255
    """
    return (x<<4)+y

ESC=chr(c(1,11))

##
## Definitions of Control Character Sequences from ECMA-48
##
C0  = "[00/00-01/15]"
C1  = "01/11,[04/00-05/15]"
CSI = "01/11,05/11,[03/00-03/15]*,[02/00-02/15]*,[04/00-07/14]"
##
## Compile single definition to regular expression
##
rx_char=re.compile(r"^(\d\d)/(\d\d)$")
rx_range=re.compile(r"^\[(\d\d)/(\d\d)-(\d\d)/(\d\d)\](\*?)$")
def compile_ecma_def(s):
    r=[]
    for token in s.split(","):
        match=rx_range.match(token)
        if match:
            c1=c(int(match.group(1)),int(match.group(2)))
            c2=c(int(match.group(3)),int(match.group(4)))
            if c1==c2:
                x=[r"\x%02x"%c1]
            elif c1<c2:
                rr=[r"\x%02x"%x for x in range(c1,c2+1)]
                x=["[%s]"%"".join(rr)]
            else:
                rr=[r"\x%02x"%x for x in range(c2,c1+1)]
                x=["[%s]"%"".join(rr)]
            if match.group(5):
                x+="*"
            r+=x
            continue
        match=rx_char.match(token)
        if match:
            r+=[r"\x%02x"%c(int(match.group(1)),int(match.group(2)))]
            continue
        raise Exception("Invalid token: <%s>"%token)
    return "".join(r)
##
## Compile ECMA-48 definitions to regular expression
##
def get_ecma_re():
    re_csi=compile_ecma_def(CSI)
    re_c1=compile_ecma_def(C1).replace("\\x5b","")
    re_c0=compile_ecma_def(C0).replace("\\x0d","").replace("\\x0a","").replace("\\x1b","").replace("\\x09","") # \n,\r, ESC, \t
    re_vt100="\x1b[c()78]" # VT100
    return "|".join(["(%s)"%r for r in (re_csi,re_c1,re_c0,re_vt100)])
##
## Remove ECMA-48 Control Sequences from a string
##
rx_ecma=re.compile(get_ecma_re())
def strip_control_sequences(s):
    """
    Normal text leaved untouched
    >>> strip_control_sequences("Lorem Ipsum")
    'Lorem Ipsum'
    
    CR,LF and ESC survive from C0 set
    >>> repr(strip_control_sequences("".join([chr(i) for i in range(32)])))
    "'\\\\n\\\\r\\\\x1b'"
    
    C1 set stripped (ESC+[ survive)
    >>> strip_control_sequences("".join(["\x1b"+chr(i) for i in range(64,96)]))
    '\\x1b['
    
    CSI without P and I stripped
    >>> strip_control_sequences("\x1b[@\x1b[a\x1b[~")
    ''
    
    CSI with I stripped
    >>> strip_control_sequences("\x1b[ @\x1b[/~")
    ''
    
    CSI with P and I stripped
    >>> strip_control_sequences("\x1b[0 @\x1b[0;7/~")
    ''
    
    Cleaned stream
    >>> strip_control_sequences("L\x1b[@or\x1b[/~em\x1b[0 @ Ips\x1b[0;7/~um\x07")
    'Lorem Ipsum'
    
    Incomplete CSI passed
    >>> strip_control_sequences("\x1b[")
    '\\x1b['
    
    Incomplete C1 passed
    >>> strip_control_sequences('\x1b')
    '\\x1b'
    """
    return rx_ecma.sub("",s)

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## SLA Probe
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.registry import Registry
##
## Probe registry
##
class ProbeRegistry(Registry):
    name="ProbeRegistry"
    subdir="probes"
    classname="Probe"
    apps=["noc.pm"]
probe_registry=ProbeRegistry()
##
## Metaclass for Probe
##
class ProbeBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        m.scripts={}
        probe_registry.register(m.name,m)
        return m
##
## SLA Probe
##
class Probe(object):
    __metaclass__=ProbeBase
    name=None
    def __init__(self,probe_name,monitor,config):
        self.probe_name=probe_name
        self.monitor=monitor
        self.factory=monitor.factory
        self.result={}
        self.config=config
    ##
    ## Run method should call feed_result
    ##
    def run(self):
        pass
    ##
    ##
    ##
    def feed_result(self,object,result):
        self.result[object]=result
        print object,result
    ##
    ##
    ##
    def get(self,name,default=None):
        if self.config.has_option(self.probe_name,name):
            return self.config.get(self.probe_name,name)
        else:
            return default



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *


class Migration:
    
    def forwards(self):
        db.add_column("peer_peeringpoint","profile_name",models.CharField("Profile",max_length=128,null=True))
        db.add_column("peer_lgquerycommand","profile_name",models.CharField("Profile",max_length=128,null=True))
        
        for id,n in db.execute("SELECT id,name FROM peer_peeringpointtype"):
            db.execute("UPDATE peer_peeringpoint SET profile_name=%s WHERE type_id=%s",[n,id])
            db.execute("UPDATE peer_lgquerycommand SET profile_name=%s WHERE peering_point_type_id=%s",[n,id])
        db.delete_column("peer_peeringpoint","type_id")
        db.delete_column("peer_lgquerycommand","peering_point_type_id")
        db.delete_table('peer_peeringpointtype')
        db.execute("ALTER TABLE peer_peeringpoint ALTER profile_name SET NOT NULL")
        db.execute("ALTER TABLE peer_lgquerycommand ALTER profile_name SET NOT NULL")
        
    
    def backwards(self):
        db.create_table('peer_peeringpointtype', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=32,unique=True))
        ))
        PeeringPointType = db.mock_model(model_name='PeeringPointType', db_table='peer_peeringpointtype', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        db.add_column("peer_peeringpoint","type",models.ForeignKey(PeeringPointType,null=True))
        db.add_column("peer_lgquerycommand","peering_point_type",models.ForeignKey(PeeringPointType,null=True))
        
        
        for pn, in db.execute("SELECT DISTINCT(profile_name) FROM peer_peeringpoint"):
            db.execute("INSERT INTO peer_peeringpointtype(name) VALUES(%s)",[pn])
            ppt_id=db.execute("SELECT id FROM peer_peeringpointtype WHERE name=%s",[pn])[0][0]
            db.execute("UPDATE peer_peeringpoint SET type_id=%s WHERE profile_name=%s",[ppt_id,pn])
            db.execute("UPDATE peer_lgquerycommand SET peering_point_type_id=%s WHERE profile_name=%s",[ppt_id,pn])
            
        db.delete_column("peer_peeringpoint","profile_name")
        db.delete_column("peer_lgquerycommand","profile_name")
        
        db.execute("ALTER TABLE peer_peeringpoint ALTER type_id SET NOT NULL")
        db.execute("ALTER TABLE peer_lgquerycommand ALTER peering_point_type_id SET NOT NULL")
        
        
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## SLA Monitor
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.daemon import Daemon
from noc.lib.nbsocket import SocketFactory
import logging,time
from noc.pm.probes import probe_registry

probe_registry.register_all()

##
## SLA Monitor Daemon
##
class SLAMonitor(Daemon):
    daemon_name="noc-sla"
    ##
    ##
    ##
    def __init__(self):
        Daemon.__init__(self)
        logging.info("Running SLA Monitor")
        self.factory=SocketFactory()
        self.probes={}
        self.init_probes()
    ##
    ##
    ##
    def init_probes(self):
        for probe_name in self.config.sections():
            if probe_name=="main":
                continue
            probe=self.config.get(probe_name,"probe")
            if not probe:
                self.error("No probe type specified for '%s'"%probe_name)
                continue
            if probe not in probe_registry.classes:
                self.error("Invalid probe type '%s' for probe '%s'"%(probe,probe_name))
                continue
            probe=probe_registry[probe]
            self.probes[probe_name]=probe(probe_name,self,self.config)
            self.probes[probe_name].run()
    ##
    ##
    ##
    def run(self):
        while True:
            self.factory.run()
            print "."



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Create "default" VC domain, if not exists
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.vc.models import *

class Migration:
    
    def forwards(self):
        if db.execute("SELECT COUNT(*) FROM vc_vcdomain WHERE name=%s",["default"])[0][0]==0:
            db.execute("INSERT INTO vc_vcdomain(name,description) VALUES(%s,%s)",["default","Default VC Domain"])
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Prefix list builder
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.peer.models import PeeringPoint
from noc.peer.resolver import Resolver
from noc.peer.tree import optimize_prefix_list

PL_THRESHOLD=10

##
## Queries whois database
## and return a list of
## (peering_point,prefix-list-name,prefix-list)
##
def build_prefix_lists():
    pl2pp={}
    f2pl={}
    result=[]
    for pp in PeeringPoint.objects.all():
        for n,f in pp.generated_prefix_lists:
            if n not in pl2pp:
                pl2pp[n]=[pp]
            else:
                pl2pp[n]+=[pp]
            f2pl[f]=n
    r=Resolver().resolve(f2pl.keys())
    for f,v in r.items():
        if f not in f2pl:
            continue
        members,prefix_list=v
        members=list(members)
        members.sort()
        if len(prefix_list)>PL_THRESHOLD:
            prefix_list=optimize_prefix_list(prefix_list)
            strict=False
        else:
            prefix_list=list(prefix_list)
            prefix_list.sort()
            strict=True
        pl_name=f2pl[f]
        for pp in pl2pp[pl_name]:
            profile=pp.profile
            pl=profile.generate_prefix_list(pl_name,prefix_list,strict)
            result+=[(pp,pl_name,pl)]
    return result
            



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Various system utils
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import ConfigParser,sys,logging,os,signal,optparse,datetime,traceback
from noc.lib.debug import error_report,frame_report,set_crashinfo_context
from noc.lib.validators import is_ipv4

# Load netifaces to resolve interface addresses when possible
try:
    import netifaces
    USE_NETIFACES=True
except ImportError:
    USE_NETIFACES=False

class Daemon(object):
    daemon_name="daemon"
    defaults_config_path="etc/%(daemon_name)s.defaults"
    config_path="etc/%(daemon_name)s.conf"
    
    LOG_LEVELS = {'debug'   : logging.DEBUG,
                  'info'    : logging.INFO,
                  'warning' : logging.WARNING,
                  'error'   : logging.ERROR,
                  'critical': logging.CRITICAL}
                  
    def __init__(self):
        # Chdir to the root of project
        os.chdir(os.path.join(os.path.dirname(sys.argv[0]),".."))
        # Parse commandline
        self.opt_parser=optparse.OptionParser()
        self.opt_parser.add_option("-c","--config",action="store",type="string",dest="config",
                help="Read config from CONFIG")
        self.opt_parser.add_option("-f","--foreground",action="store_false",dest="daemonize",default=True,
                help="Do not daemonize. Run at the foreground")
        self.setup_opt_parser()
        self.options,self.args=self.opt_parser.parse_args()
        if len(self.args)<1 or self.args[0] not in ["start","stop","refresh"]:
            self.opt_parser.error("You must supply one of start|stop|refresh commands")
        # Read config
        self.config=None
        self.load_config()
        # Set up logging
        if self.config.get("main","loglevel") not in self.LOG_LEVELS:
            raise Exception("Invalid loglevel '%s'"%self.config.get("main","loglevel"))
        loglevel=self.LOG_LEVELS[self.config.get("main","loglevel")]
        for h in logging.root.handlers:
            logging.root.removeHandler(h) # Dirty hack for baseConfig
        if self.options.daemonize:
            if self.config.get("main","logfile"):
                logging.basicConfig(level=loglevel,
                                filename=self.config.get("main","logfile"),
                                format='%(asctime)s %(levelname)s %(message)s',
                                filemode="a+")
        else:
            logging.basicConfig(level=logging.DEBUG,format='%(asctime)s %(levelname)s %(message)s')
        # Register signal handlers if any
        for s in [s for s in dir(self) if s.startswith("SIG")]:
            try:
                sig=getattr(signal,s)
            except AttributeError:
                logging.error("Signal '%s' is not supported on this platform"%s)
                continue
            signal.signal(sig,getattr(self,s))
    ##
    ##
    ##
    def load_config(self):
        first_run=True
        if self.config:
            logging.info("Loading config")
            first_run=False
        self.config=ConfigParser.SafeConfigParser()
        if self.defaults_config_path:
            self.config.read(self.defaults_config_path%{"daemon_name":self.daemon_name})
        if self.options.config:
            self.config.read(self.options.config)
        elif self.config_path:
            self.config.read(self.config_path%{"daemon_name":self.daemon_name})
        if not first_run:
            self.on_load_config()
        if self.config.get("main","logfile"):
            set_crashinfo_context(self.daemon_name,os.path.dirname(self.config.get("main","logfile")))
        else:
            set_crashinfo_context(None,None)
    ##
    ## Called after config reloaded by SIGHUP.
    ##
    def on_load_config(self):
        pass
    ##
    ## Main daemon loop. Should be overriden
    ##
    def run(self):
        pass

    def become_daemon(self):
        try:
            if os.fork():
                sys.exit(0)
        except OSError,e:
            sys.stderr.write("Fork failed")
            sys.exit(1)
        os.setsid()
        os.umask(022)
        try:
            pid=os.fork()
        except OSError,e:
            sys.stderr.write("Fork failed")
            os._exit(1)
        if pid:
            pidfile=self.config.get("main","pidfile")
            if pidfile:
                f=open(pidfile,"w")
                f.write(str(pid))
                f.close()
            os._exit(0)
        i=open("/dev/null","r")
        o=open("/dev/null","a+")
        e=open("/dev/null","a+")
        os.dup2(i.fileno(), sys.stdin.fileno())
        os.dup2(o.fileno(), sys.stdout.fileno())
        os.dup2(e.fileno(), sys.stderr.fileno())
        sys.stdout=o
        sys.stderr=e
    ##
    ## Returns IP address for IP addresses or interface names
    ##
    def resolve_address(self,s):
        if is_ipv4(s):
            return s
        if USE_NETIFACES:
            try:
                a=netifaces.ifaddresses(s)
            except ValueError:
                raise Exception("Invalid interface '%s'"%s)
            try:
                return a[2][0]["addr"]
            except (IndexError,KeyError):
                raise Exception("No ip address for interface: '%s' found"%s)
        raise Exception("Cannot resolve address '%s'"%s)
    ##
    ## Add additional options to setup_opt_parser
    ##
    def setup_opt_parser(self):
        pass
    ##
    ## Process self.args[0] command
    ##
    def process_command(self):
        getattr(self,self.args[0])()
    ##
    ## "start" command handler
    ##
    def start(self):
        # Daemonize
        if self.options.daemonize:
            self.become_daemon()
        try:
            self.run()
        except KeyboardInterrupt:
            pass
        except:
            error_report()
    ##
    ## "stop" command handler
    ##
    def stop(self):
        pidfile=self.config.get("main","pidfile")
        if os.path.exists(pidfile):
            f=open(pidfile)
            pid=int(f.read().strip())
            f.close()
            logging.info("Stopping %s pid=%s"%(self.daemon_name,pid))
            try:
                os.kill(pid,signal.SIGKILL)
            except:
                pass
            os.unlink(pidfile)
    ##
    ## "refresh" command handler
    ##
    def refresh(self):
        self.stop()
        self.start()
    ##
    ## Dump current execution frame trace on SIGUSR2
    ##
    def SIGUSR2(self,signo,frame):
        frame_report(frame)
    ##
    ## Reload config on SIGHUP
    ##
    def SIGHUP(self,signo,frame):
        self.load_config()
